##!/home/bpeek/anaconda3/bin/python

import sys
import time as ti
from netCDF4 import Dataset
import numpy as np
from datetime import datetime

# timer start for testing last line of script is elapsed time
start_time = ti.time()

today = datetime.today()

# Here is a list of all data sets that have time associated with them
fields = ['azimuth', 'elevation', 'Reflectivity', 'Average_Velocity', 'pitch',\
          'roll', 'heading', 'drift', 'latitude', 'longitude', 'altitude',\
          'northward_velocity', 'eastward_velocity', 'tx', 'sbm_counter', \
          'frequency', 'pulse_width', 'prt', 'pulse_band_width', \
          'rg_width', 'pulse_num', 'batch_band_width', 'rotation', 'tilt', \
          'vertical_velocity', 'northward_wind', 'eastward_wind', 'vertical_wind', \
          'heading_rate', 'pitch_rate', 'roll_rate', 'HWX_Range_Doppler_Map', \
          'power_reference']

hist = 'All data sets sorted for time order by sortdata.py. Modified: '
desc = 'All data sets sorted for time order to correct time issues'

filename = sys.argv[1]
print("CFRadial file: ", filename)
fh = Dataset(filename, 'a')
time = fh.variables['time'][:]

# Add description and history to global attributes
def add_global_atts(fh):
	fh.description = desc
	fh.history = hist +today.strftime("%d-%b-%Y")

# Here's where the magic happens, the data gets sorted 
def sort_list(list1, list2):
    zipped_pairs = zip(list2, list1)
    z = [x for _, x in sorted(zipped_pairs)]
    return z

def process_data(var):
	print("ProcessData: ", var)
	var1 = fh.variables[var][:]
	fh.variables[var][:] = sort_list(var1, time)

varh = 'radar_beam_width_h'
varv = 'radar_beam_width_v'
def add_variables(fh):
	if varh in fh.variables.keys():
		print("var: radar_beam_width_h exists")
	else:
		print("Adding radar_beam_width_h")
		radar_beam_width_h = fh.createVariable('radar_beam_width_h', np.float32)
		radar_beam_width_h.long_name = 'half_power_radar_beam_width_h_channel' ;
		radar_beam_width_h.units = 'degrees'
		radar_beam_width_h.meta_group = 'radar_parameters'
		radar_beam_width_h[:] = 1.4
	if varv in fh.variables.keys():
		print("var: radar_beam_width_v exists")
	else:
		print("Adding radar_beam_width_v")
		radar_beam_width_v = fh.createVariable('radar_beam_width_v', np.float32)
		radar_beam_width_v.long_name = 'half_power_radar_beam_width_v_channel' ;
		radar_beam_width_v.units = 'degrees' ;
		radar_beam_width_v.meta_group = 'radar_parameters' ;
		radar_beam_width_v[:] = 5.6 ;

if hist in fh.history:
	print("CFRadial file: ", filename, " Was previously Sorted")
else:
	print("Will be Sorted")
	for field in fields:
		process_data(field)
	
# Process time, the variable 'time', must be done last so the correct idicies are used for the other variables 
fh.variables['time'][:] = sorted(time)

# Here we add antenna beam width h&v, so you can use Radx2Grid
add_variables(fh)

# Add desciption and history to global attributes 
add_global_atts(fh)

# Close the file
fh.close()

# Elapsed time to process file
print("ElapsedTime", ti.time() - start_time)


