      PROGRAM LEASTWIND
C
C     MAIN PROGRAM OF WIND_INTERPOLATE_RT_AUTO_FIRST_DEFRECKLE
C       
C ... FUNCTION:   
C       
C          THIS PROGRAM COMPUTES SUMMATIONS OF PRODUCTS NEEDED FOR THE
C          FIRST LEAST SQUARES WIND SOLUTION. THE GOVERNING EQUATIONS 
C          ARE FORMULATED IN R-THETA-Z COORDINATES RELATIVE TO THE STORM
C          CENTER, AND REPRESENT THE AZIMUTHAL STRUCTURE IN TERMS OF 
C          FOURIER WAVENUMBER 0 AND 1 COMPONENTS.  AIRCRAFT MOTION 
C          REMOVAL, BARGEN-BROWN DEALIASING, DEFRECKLING, AND SURFACE
C          REMOVAL ARE PERFORMED AS PART OF THE QC PROCESS. 
C       
C ... LEASTWIND SUBPROGRAM REFERENCES:      
C       
C          NON-SYSTEM     OPENRAMFILE, ABCD, WRITEABCD
C
C          SYSTEM         GETARG
C 
C ... IMPORTANT FLAGS: <()> INDICATES DEFAULT
C          
C          IGRAPHICS      USE GRAPHICS (0) NO (1) YES
C          COMYES         USE COMMAND FILE (N) NO <(Y)> YES
C          IACROSS        AZI-COVERAGE (0) SECTOR <(1)> FULL
C          IGUESS         USE A GUESS WIND FIELD <(0)> NO (1) YES
C          IREVERSE       REVERSE SIGN OF DOPPLER VR <(0)> NO (1) YES
C          IREADABCD      READ IN SUMFILE <(0)> NO (1) YES
C          IWRITEABCD     WRITE OUT SUMFILE (0) NO <(1)> YES
C          IVSPEED        USE CONST. AZI ADV. <(0)> NO (1) YES
C          IRAMSFILES     USE RAMSFILE <(0)> NO (1) YES
C      
C     VERSION:  <TBD>
C       
C     CODE WRITTEN BY:  JOHN GAMACHE   
C                       HRD/AOML/NOAA          
C                       MIAMI, FL 33149      
C       
C     FOR ADDITIONAL DETAILS ON THE ALGORITHM SEE TBD
C       
C     BASED ON THEORY BY:  <TBD>
C       
C     REFERENCE THE BOOK:  <TBD>
C       
C     **************************************************  
C     *               IMPORTANT NOTE                   *  
C     *                                                *  
C     *                    <TBD>                       *  
C     *                                                *  
C     **************************************************  
C       
C     SPECIFICATIONS FOR LOCAL VARIABLES
C       
C **** HEADER RECORD OF AN HRD-TYPE DOPPLER WIND FILE 
C 
      CHARACTER G4CONTINUE*1,GROUNDYES*1
      LOGICAL BLANKEXISTS
      REAL P3RADMIN,RADRAT
      LOGICAL I_EXIST
      INTEGER IB,JB,KB,IE,JE,KE,IH,IM,IS,NKSIP,ILEN,IRADRAT
      INTEGER IMAX,JMAX,KMAX,IDBZADD,IRADMASK
      REAL STIME,ETIME,OLAT,OLON,SX,SY,SZ,XZ,YZ,ZZ,ROT
      REAL SMOTIONU,SMOTIONV,RADMASK
      REAL REALTOL
      CHARACTER FLTNAME*8,EXPERIMENT*32,MASKADDFILE*10
      CHARACTER FILNAM(40)*80,AIRCHAR*1
C 
C **** REMAINING 
C 
      REAL SHORTPULSEADDDB,SHORTPULSEADDRADIUS
      LOGICAL SHORTPULSEADDEXISTS
      INTEGER N,IJK,IROLLTOL
      INTEGER KMIN2,KMAX2
      INTEGER LUIN,LUOUT,LUSCAN,LUW,NFILE,NF,NFF
      INTEGER IGRAPHICS,IACROSS,IGUESS,IREVERSE,IREADABCD
      INTEGER IWRITEABCD,IVSPEED,IRAMSFILES,IERR
      PARAMETER(N1=6000000,N2=12000000,N3=18000000,N8=48000000)
      REAL TSTART,TEND,SLOPE1,CENTIME
      REAL TSTARTARRAY(40),TENDARRAY(40)
      REAL XSHIFT(80),YSHIFT(80),DBZFACT(80)
      REAL RRMAX,RTMAX,RZMAX,ERMAX,ETMAX,EZMAX
      REAL XACC,YACC,ZACC,PIMAX
      REAL SINTOL,CTR,RFLAG,DATARATE
      REAL DBZTEST,RRRR,VSPEED0,VFACTOR
      REAL,ALLOCATABLE::SUM1(:),SUM2(:),SUM3(:)
      REAL,ALLOCATABLE::SUM4(:),SUM5(:),SUM6(:)
      REAL,ALLOCATABLE::SUM7(:),SUM8(:),SUM9(:)
      REAL,ALLOCATABLE::SUM10(:),SUM11(:),SUM12(:),SUM13(:)
      REAL,ALLOCATABLE::SUM14(:),SUM15(:),SUM16(:)
      REAL,ALLOCATABLE::SUM17(:),SUM18(:),SUM19(:)
      REAL,ALLOCATABLE::SUM20(:),SUM21(:),SUM22(:)
      REAL,ALLOCATABLE::SUM23(:),SUM24(:)
      REAL,ALLOCATABLE::DISTXYZ(:)
      REAL,ALLOCATABLE::SUMWTSAVE(:),SUMDBZ(:)
      REAL,ALLOCATABLE::VELS(:)
      REAL,ALLOCATABLE::UGUESS(:),VGUESS(:),WGUESS(:)
      CHARACTER TATF(80)*2,YESNO*1
      CHARACTER COMYES*1,GRAPHICSYES*1,GRAPHICSYES2*2,GRAPHICSYES3*3
      CHARACTER OSUMFILENAME(80)*80,RAMSFILES(80)*80
C     N1 IS MAXIMUM VALUES OF IMAX*JMAX*KMAX--SEE THEIR DEFINITIONS BELOW
C     N2, N3, AND N8 ARE SIMPLY N1 * 2, 3 OR 8
C     N1, N2, N3, AND N8 ARE USED TO ALLOCATE THE LARGE ARRAYS WHICH HAVE MULTIPLE DIMENSIONS INS SUBROUTINES
C
C ... DESCRIPTION OF VARIABLES PASSED TO SUBROUTINES
C       
C     RAMSFILES            - RAMSFILE NAME
C     LUOUT                - LOGICAL UNIT NUMBER TO DISPLAY MESSAGES
C     STIME,ETIME          - START,END TIME (S)
C     DATARATE             - FILE DATA RATE
C     IERR                 - ERROR CONDITION FLAG
C     RRMAX,RTMAX,RZMAX    - RADIAL,AZI,VERTICAL SCALE OF INFLUENCE (KM,DEG,KM)
C     ERMAX,ETMAX,EZMAX    - RADIAL,AZI,VERTICAL E-FOLDING DISTANCE (KM,DEG,KM)
C     XACC,YACC,ZACC       - MAX ALLOWED RADIAL,AZI,VERTICAL DIST. BETWEEN AVG. OBS POS. AND GRID POINT
C     NF                   - FILE NO.
C     SUM1-13              - SUMS OF PRODUCTS ACCUMULATED DURING INTERP. PROCESS
C                            See ABCD subroutines for definition of sum1-sum13
C     KMIN2,KMAX2          - TYPICALLY MIN AND MAX VERTICAL LEVEL NO.
C     SINTOL               - SIN OF MAX ALLOWED DOPPLER RADIAL ANGLE FROM HORIZ.
C                            This is now generally 90 degrees, thus including all radials
C     SX,SY,SZ             - RADIAL,AZI,VERTICAL GRID SPACING (KM,DEG,KM) 
C                            of gridded files
C     XZ,YZ,ZZ             - MIN RADIUS,AZI,HEIGHT (KM,DEG,KM)
C                            of gridded files
C     ROT                  - COORD. ROTATION (ALWAYS ZERO IN CYL COORDS)
C     IMAX,JMAX,KMAX       - RADIAL,AZI,VERTICAL ARRAY DIMENSION
C     SUMDBZ               - WEIGHTED AVG. REFLECTIVITY AT GRIDPOINT (I,J,K) FOR FORE OR AFT SCAN
C                          - during interpolation SUMDBZ is an accumulator
C     SUMWTSAVE            - ACCUMULATED WEIGHT AT GRIDPOINT (I,J,K) FOR FORE OR AFT SCAN
C     VELS                 - WEIGHTED AVG. R,THETA,Z DIST. BETWEEN DATUM AND GRIDPOINT
C     OLAT,OLON            - LAT,LON OF GRID ORIGIN (STORM CENTER)
C     SLOPE1               - REFLECTIVITY SLOPE CORRECTION FACTOR SET TO ONE
C     CENTIME              - ANALYSIS TIME (S)
C     TSTART,TEND          - SEE STIME,ETIME
C     SMOTIONU,SMOTIONV    - STORM MOTION (M/S)
C     FLTNAME              - FLIGHT NAME
C     DBZTEST              - MIN DBZ TO INCLUDE
C     RRRR                 - MIN RANGE TO INCLUDE 
C     UGUESS,VGUESS,WGUESS - GUESS RADIAL, TANGENTIAL, AND VERTICAL WIND (ZERO, M/S)
C     DISTXYZ              - DISTANCE OF THE CLOSEST OBS THAT WAS INTERP. TO GRID POINT (I,J,K)
C     PIMAX                - MAX AZI ANGLE FOR OBS TO BE INCLUDED IN WT AVG
C     VSPEED0              - CONST. ADV. SPEED USED IN COMPOSITING (M/S)
C     VFACTOR              - FACTOR MULT. WIND SPEED TO USE IN COMPOSITING (M/S)
C     TATF                 - IDENTIFY SCAN TYPE 
C     LUW                  - LOGICAL UNIT NUMBER FOR OUTPUT SUMFILE
C     NFF                  - FORE OR AFT' FILE NO.
C    
      SHORTPULSEADDDB=-1.
      SHORTPULSEADDRADIUS=-1.
      INQUIRE(FILE='short_pulse_add',EXIST=SHORTPULSEADDEXISTS)
      IF(SHORTPULSEADDEXISTS)THEN
         OPEN(88,FILE='short_pulse_add')
         READ(88,*)SHORTPULSEADDDB
         WRITE(6,*)'SHORTPULSEADDDB = ',SHORTPULSEADDDB
         READ(88,*)SHORTPULSEADDRADIUS
         WRITE(6,*)'SHORTPULSEADDRADIUS = ',SHORTPULSEADDRADIUS
         CLOSE(88)
         PAUSE
      ENDIF
      LUOUT=6
      LUSCAN=97      
      CTR=3.14159/180.
      RFLAG=-1.0E+10
C 
      OPEN(1,FILE='/dev/null')
C 
C ... INITIALIZE DBZ VECTOR TO FLAGGED VALUE
C 
      ALLOCATE(SUM1(N1))
      ALLOCATE(SUM2(N1))
      ALLOCATE(SUM3(N1))
      ALLOCATE(SUM4(N1))
      ALLOCATE(SUM5(N1))
      ALLOCATE(SUM6(N1))
      ALLOCATE(SUM7(N1))
      ALLOCATE(SUM8(N1))
      ALLOCATE(SUM9(N1))
      ALLOCATE(SUM10(N1))
      ALLOCATE(SUM11(N1))
      ALLOCATE(SUM12(N1))
      ALLOCATE(SUM13(N1))
      ALLOCATE(SUM14(N1))
      ALLOCATE(SUM15(N1))
      ALLOCATE(SUM16(N1))
      ALLOCATE(SUM17(N1))
      ALLOCATE(SUM18(N1))
      ALLOCATE(SUM19(N1))
      ALLOCATE(SUM20(N1))
      ALLOCATE(SUM21(N1))
      ALLOCATE(SUM22(N1))
      ALLOCATE(SUM23(N1))
      ALLOCATE(SUM24(N1))
      ALLOCATE(DISTXYZ(N8))
      ALLOCATE(SUMWTSAVE(N1))
      ALLOCATE(SUMDBZ(N1))
      ALLOCATE(UGUESS(N1))
      ALLOCATE(VGUESS(N1))
      ALLOCATE(WGUESS(N1))
      ALLOCATE(VELS(N3))
      DO N=1,N1
       SUMDBZ(N)=RFLAG
      ENDDO
C 
C ... PROCESS ALL ARGUMENTS IN CALL TO PROGRAM
C
C ... CHECK FOR GRAPHICS OPTION AND SET FLAG
C 
      INQUIRE(FILE='blankradius',EXIST=BLANKEXISTS)
      IF(BLANKEXISTS)THEN
         OPEN(87,FILE='blankradius')
         READ(87,*)P3RADMIN
         WRITE(6,*)'There is a blankradius file that will eliminate'
         WRITE(6,*)'superobs in radius less than',P3RADMIN
         WRITE(6,*)'Do you wish to continue, if so, enter "y" or "Y"'
c         READ(5,'(A1)')G4CONTINUE
c         IF(G4CONTINUE.NE.'y'.AND.G4CONTINUE.NE.'Y')STOP
      ELSE
         P3RADMIN=0.01
      ENDIF
      WRITE(6,*)'P3RADMIN = ',P3RADMIN
      CALL GETARG(1,GRAPHICSYES)
      IF(GRAPHICSYES.EQ.'G'.OR.GRAPHICSYES.EQ.'g')THEN
       IGRAPHICS=1
      ELSE
       IGRAPHICS=0
      ENDIF
C
C ... SET WHETHER THERE IS A BACKGROUND GUESS FIELD
C ... IN THE CASE OF WAVENUMBER 0 PLUS MEAN ONLY GENERALLY THIS WILL ALWAYS BE IGUESS = 0
C
      CALL GETARG(2,GRAPHICSYES)
      IF(GRAPHICSYES.EQ.'G'.OR.GRAPHICSYES.EQ.'g')THEN
       IGUESS=1
      ELSE
       IGUESS=0
      ENDIF
C
C ... IPLANE = 1 INDICATES P3, IPLANE = 2  INDICATES G-IV
C
      CALL GETARG(3,GRAPHICSYES)
      IF(GRAPHICSYES.EQ.'G'.OR.GRAPHICSYES.EQ.'g')THEN
       IPLANE=2
       WRITE(6,*)'IPLANE = ',IPLANE
      ELSE
       IPLANE=1
       WRITE(6,*)'IPLANE = ',IPLANE
      ENDIF
C
C ... DETERMINE GAP ALLOWED IN BARGEN-BROWN DE-ALIASING IF THE GAPS IS LARGER, 
C ... THEN ALL VALUES OF DOPPLER RADIAL VELOCITY BEYOND GAP ARE FLAGGED
C
      CALL GETARG(4,GRAPHICSYES2)
      READ(GRAPHICSYES2,'(I2)')ISTORMLEVEL
C
C ... DETERMINE WHETHER THIS IS RESEARCH OR OPERATIONAL...FOR NOW THIS PARAMETER MAKES NO DIFFERENCE
C
      CALL GETARG(5,GRAPHICSYES)
      IF(GRAPHICSYES.EQ.'R'.OR.GRAPHICSYES.EQ.'r')THEN
       IREALTIME=1
      ELSE
       IREALTIME=0
      ENDIF
      CALL GETARG(6,GRAPHICSYES)
      IF(GRAPHICSYES.EQ.'V'.OR.GRAPHICSYES.EQ.'v')THEN
       IVARIATIONAL=1
      ELSE
       IVARIATIONAL=0
      ENDIF
      CALL GETARG(7,GRAPHICSYES)
      IF(GRAPHICSYES.EQ.'A'.OR.GRAPHICSYES.EQ.'a')THEN
       IAFTFORE=1
      ELSEIF(GRAPHICSYES.EQ.'F'.OR.GRAPHICSYES.EQ.'f')THEN
       IAFTFORE=2
      ELSE
       IAFTFORE=3
      ENDIF
      CALL GETARG(8,GRAPHICSYES)
      IF(GRAPHICSYES.EQ.'5')THEN
       ISIGMET=5
      ELSE
       ISIGMET=8
      ENDIF
      WRITE(6,*)'ISTORMLEVEL = ',ISTORMLEVEL
      CALL GETARG(9,GRAPHICSYES2)
      READ(GRAPHICSYES2,'(I2)')IROLLTOL
      ROLLTOL=IROLLTOL
      CALL GETARG(10,GRAPHICSYES2)
      READ(GRAPHICSYES2,'(I2)')NSKIP
      ROLLTOL=IROLLTOL
      CALL GETARG(11,GRAPHICSYES3)
      READ(GRAPHICSYES3,'(I3)')IDBZADD
      CALL GETARG(12,GRAPHICSYES3)
      ILEN=LEN_TRIM(GRAPHICSYES3)
      READ(GRAPHICSYES3,'(I3)')IDBZMASK
      IF(IPLANE.EQ.1.AND.ILEN.EQ.0)THEN
         IF(IAFTFORE.EQ.1)THEN
            IDBZADD=0.
            IDBZMASK=20.
         ELSE
            IDBZADD=5.
            IDBZMASK=25.
         ENDIF
      ELSEIF(ILEN.EQ.0)THEN
         IDBZADD=0.
         IDBZMASK=20.
      ENDIF
      WRITE(6,*)'IDBZMASK,IDBZADD,ILEN = ',IDBZMASK,IDBZADD,ILEN
      CALL GETARG(13,GRAPHICSYES3)
      ILEN=LEN_TRIM(GRAPHICSYES3)
      IF(ILEN.EQ.0)THEN
         DOPMIN=-90.
         DOPMAX=90.
         DBZMIN=-20.
         DBZMAX=60.
      ELSE
       READ(GRAPHICSYES3,'(I3)')IDOPMIN
       DOPMIN=IDOPMIN
       CALL GETARG(14,GRAPHICSYES3)
       READ(GRAPHICSYES3,'(I3)')IDOPMAX
       DOPMAX=IDOPMAX
       CALL GETARG(15,GRAPHICSYES3)
       READ(GRAPHICSYES3,'(I3)')IDBZMIN
       DBZMIN=IDBZMIN
       CALL GETARG(16,GRAPHICSYES3)
       READ(GRAPHICSYES3,'(I3)')IDBZMAX
       DBZMAX=IDBZMAX
      ENDIF
      CALL GETARG(17,GRAPHICSYES)
      ILEN=LEN_TRIM(GRAPHICSYES)
      IF(ILEN.EQ.0)THEN
         IZEROWIND=0
      ELSE
         READ(GRAPHICSYES,'(A1)')YESNO
         IF(YESNO.EQ.'Y'.OR.YESNO.EQ.'y')THEN
            IZEROWIND=1
         ELSE
            IZEROWIND=0
         ENDIF
      ENDIF
      CALL GETARG(18,GRAPHICSYES)
      ILEN=LEN_TRIM(GRAPHICSYES)
      IF(ILEN.EQ.0)THEN
         IKLUGETEST=2
      ELSE
         READ(GRAPHICSYES,'(I1)')IKLUGETEST
      ENDIF
      CALL GETARG(19,GRAPHICSYES)
      ILEN=LEN_TRIM(GRAPHICSYES)
      IF(ILEN.EQ.0)THEN
         IREMOVEGROUND=0
      ELSE
         READ(GRAPHICSYES,'(A1)')GROUNDYES
         IF(GROUNDYES.EQ.'G'.OR.GROUNDYES.EQ.'g')THEN
            IREMOVEGROUND=1
         ELSE
            IREMOVEGROUND=0
         ENDIF
      ENDIF
      CALL GETARG(20,GRAPHICSYES)
      ILEN=LEN_TRIM(GRAPHICSYES)
      IF(ILEN.EQ.0)THEN
         IUNFLD=0
      ELSE
         READ(GRAPHICSYES,'(A1)')GROUNDYES
         IF(GROUNDYES.EQ.'U'.OR.GROUNDYES.EQ.'u')THEN
            IUNFLD=1
         ELSE
            IUNFLD=0
         ENDIF
      ENDIF
      CALL GETARG(21,GRAPHICSYES)
      ILEN=LEN_TRIM(GRAPHICSYES)
      IF(ILEN.EQ.0)THEN
         IAZIUNFLD=0
      ELSE
         READ(GRAPHICSYES,'(A1)')GROUNDYES
         IF(GROUNDYES.EQ.'a'.OR.GROUNDYES.EQ.'A')THEN
            IAZIUNFLD=1
         ELSE
            IAZIUNFLD=0
         ENDIF
      ENDIF
      CALL GETARG(22,GRAPHICSYES)
      ILEN=LEN_TRIM(GRAPHICSYES)
      IF(ILEN.EQ.0)THEN
         IDESPECKLE=0
      ELSE
         READ(GRAPHICSYES,'(A1)')GROUNDYES
         IF(GROUNDYES.EQ.'D'.OR.GROUNDYES.EQ.'d')THEN
            IDESPECKLE=1
         ELSE
            IDESPECKLE=0
         ENDIF
      ENDIF
      CALL GETARG(23,GRAPHICSYES)
      ILEN=LEN_TRIM(GRAPHICSYES)
      IF(ILEN.EQ.0)THEN
         RADRAT=4.
      ELSE
         READ(GRAPHICSYES,'(I1)')IRADRAT
         RADRAT=FLOAT(IRADRAT)
      ENDIF
      CALL GETARG(24,GRAPHICSYES2)
      ILEN=LEN_TRIM(GRAPHICSYES2)
      IF(ILEN.EQ.0)THEN
         RADMASK=10.
      ELSE
         READ(GRAPHICSYES2,'(I2)')IRADMASK
         RADMASK=FLOAT(IRADMASK)
      ENDIF
      CALL GETARG(25,GRAPHICSYES)
      WRITE(6,*)GRAPHICSYES
c      PAUSE
      ILEN=LEN_TRIM(GRAPHICSYES)
      IF(ILEN.EQ.0)THEN
         IFILLGAP=0
      ELSEIF(GRAPHICSYES.EQ.'F'.OR.GRAPHICSYES.EQ.'f')THEN
         IFILLGAP=1
         WRITE(6,*)'FLIGHT-LEVEL WIND GAPS WILL BE FILLED'
      ELSE
         IFILLGAP=0
      ENDIF
      WRITE(6,*)'IFILLGAP = ',IFILLGAP
      WRITE(6,*)'GRAPHICSYES,IUNFLD = ',GRAPHICSYES,IUNFLD
      COMYES='Y'
      IF(COMYES.EQ.'Y'.OR.COMYES.EQ.'y')THEN
       LUIN=16
       FILNAM(1)='jobfile_radar_rt'
       OPEN(16,FILE=FILNAM(1),STATUS='OLD')
      ELSE
       LUIN=1
      ENDIF
C
C **** FLIGHT NAME
C 
      READ(LUIN,'(A8)')FLTNAME
      WRITE(6,1501)FLTNAME
1501  FORMAT('FLIGHT ID IS ',A8)
      READ(FLTNAME,'(6X,A1)')AIRCHAR
      WRITE(6,*)'AIRCHAR IS ',AIRCHAR
      IF(AIRCHAR.EQ.'H')THEN
         MASKADDFILE='maskadd_42'
      ELSEIF(AIRCHAR.EQ.'I')THEN
         MASKADDFILE='maskadd_43'
      ELSEIF(AIRCHAR.EQ.'N')THEN
         MASKADDFILE='maskadd_49'
      ELSE
         MASKADDFILE='nomaskfile'
      ENDIF
      WRITE(6,*)'MASKADDFILE = ',MASKADDFILE
      INQUIRE(FILE=MASKADDFILE,EXIST=I_EXIST)
      IF(I_EXIST.AND.ILEN.EQ.0)THEN
         WRITE(6,*)'maskadd exists'
         OPEN(85,FILE=MASKADDFILE)
         READ(85,*)DBZADDA,DBZMASKA,DBZADDF,DBZMASKF
         CLOSE(85)
         IF(IAFTFORE.EQ.1)THEN
            IDBZADD=NINT(DBZADDA)
            IDBZMASK=NINT(DBZMASKA)
         ELSE
            IDBZADD=NINT(DBZADDF)
            IDBZMASK=NINT(DBZMASKF)
         ENDIF
      ELSE
         WRITE(6,*)'maskadd does not exist'
      ENDIF
      WRITE(6,*)'DBZADDA,DBZMASKA,DBZADDF,DBZMASKF = ',
     +           DBZADDA,DBZMASKA,DBZADDF,DBZMASKF
      WRITE(6,*)'FINAL IDBZADD,IDBZMASK = ',IDBZADD,IDBZMASK
C
C ... READ IN PARAMETERS AND FLAGS
C
C **** COMMAND FILE FLAG AND FILE NAME
C 
C
C **** EXPERIMENT NAME (32 CHARACTERS OR LESS)
C 
      READ(LUIN,'(A32)')EXPERIMENT
      WRITE(6,1502)EXPERIMENT
1502  FORMAT('EXPERIMENT NAME IS ',A32)
C
C **** NUMBER OF DOP3D FILES
C 
      NFILE=2
      WRITE(6,1503)NFILE
1503  FORMAT('NUMBER OF UF FILES TO BE INTERPOLATED IN THIS RUN '/
     + 'BE ',I3)
      IF(NFILE.GT.40)THEN
       WRITE(6,*)'Too many files'
       STOP
      ENDIF
C
C **** SUMMATION FILE NAMES
C 
      OSUMFILENAME(1)='SUMFILE_aft_rt_auto_first'
      OSUMFILENAME(2)='SUMFILE_fore_rt_auto_first'
C
C **** BEGIN AND END TIMES
C 
      READ(LUIN,'(3I2,X,3I2)')IB,JB,KB,IE,JE,KE
      WRITE(6,1506)NF,IB,JB,KB
1506  FORMAT('BEGINNING TIME FOR FILE NO. ',I3,' WILL BE ',3I2)
      WRITE(6,1507)NF,IE,JE,KE
1507  FORMAT('ENDING TIME FOR FILE NO. ',I3,' WILL BE ',3I2)
      TSTART=IB*3600.+JB*60+KB
      STIME=TSTART
      ETIME=IE*3600.+JE*60.+KE
      IF(ETIME.LT.STIME)THEN
       TEND=ETIME+86400.
      ELSE
       TEND=ETIME
      ENDIF
      WRITE(6,*)'STIME,ETIME,TSTART,TEND = ',
     +            STIME,ETIME,TSTART,TEND 
C
C     SUMMARY LOOP 
C            
      DO NF=1,2
       TATF(NF)='TA'
       TATF(NF)='TF'
       TSTARTARRAY(NF)=TSTART
       TENDARRAY(NF)=TEND
       DBZFACT(NF)=1.
       XSHIFT(NF)=0.
       YSHIFT(NF)=0.
       WRITE(6,1508)NF,OSUMFILENAME(NF)
1508   FORMAT('OUTPUT INTERPOLATION SUMFILE FOR INPUT FILE NO. ',I3/
     + 'WILL BE ',A80)
       WRITE(6,11508)TATF(NF)
11508  FORMAT('SCAN TYPE WILL BE ',A2)
      ENDDO
C
C **** E-W AND N-S STORM MOTION
C 
      READ(LUIN,*)SMOTIONU,SMOTIONV
      WRITE(6,1509)SMOTIONU,SMOTIONV
1509  FORMAT('ADVECTION FOR VELOCITY FOR INTERPOLATION COMPOSITE '/
     + 'WILL BE ',2F8.3,' M/S')
C
C **** COMPOSITE TIME
C 
      READ(LUIN,'(3I2)')IH,IM,IS
      WRITE(6,1510)IH,IM,IS
1510  FORMAT('TIME OF ZERO ADVECTION IN COMPOSITE WILL BE ',3I2)
      CENTIME=IH*3600+IM*60+IS
      IF(CENTIME.LT.STIME.AND.ETIME.GT.86400)CENTIME=CENTIME+86400.
C
C **** LATITUDE AND LONGITUDE OF ORIGIN
C 
      READ(LUIN,*)OLAT,OLON
      WRITE(6,1511)IH,IM,IS,OLAT,OLON
1511  FORMAT('LATITUDE AND LONGITUDE OF ORIGIN AT TIME ',3I2/
     + 'WILL BE ',2F8.3)
C
C **** RADIAL, AZIMUTHAL, AND VERTICAL RESOLUTIONS
C 
      READ(LUIN,*)SX,SY,SZ
      WRITE(6,1512)SX,SY,SZ
1512  FORMAT('RADIAL-, AZIMUTHAL-, AND VERTICAL-DIRECTION GRID '/
     + 'RESOLUTIONS ARE ',F6.2,' KM, ',F6.2,' DEGREES, AND '/
     + F6.2,' KM, RESPECTIVELY')
C
C **** MINIMUM RADIUS FROM STORM CENTER, MINIMUM AZIMUTH CCW FROM
C **** EAST, AND LOWEST HEIGHT
C 
      READ(LUIN,*)XZ,YZ,ZZ
C 
C     ROTATION IS SET TO ZERO SINCE CYLINDRICAL COORDINATES
C 
      ROT=0.
      WRITE(6,1513)XZ
1513  FORMAT('MINIMUM RADIUS FROM STORM CENTER WILL BE ',F8.3)
      WRITE(6,1514)YZ
1514  FORMAT('MINIMUM AZIMUTH COUNTERCLOCKWISE FROM EAST WILL BE ',
     + F8.3,' DEGREES')
      WRITE(6,1515)ZZ
1515  FORMAT('HEIGHT OF LEVEL 1 WILL BE ',F8.3,' KM')
C
C **** RADIAL, AZIMUTHAL, AND VERTICAL DIMENSIONS
C 
      READ(LUIN,*)IMAX,JMAX,KMAX
      WRITE(6,1516)IMAX,JMAX,KMAX
1516  FORMAT('RADIAL, AZIMUTHAL, AND VERTICAL DIMENSIONS OF ARRAYS '/
     + 'WILL BE ',3I4)
      DO IJK=1,10
       READ(LUIN,*)DUMMYIJK
      ENDDO
      READ(LUIN,*)HB2,DPB2
      READ(LUIN,*)ZLOW,ZHIGH
C
C **** RADIAL, AZIMUTHAL, AND VERTICAL RADII (KM) OF INFLUENCE
C 
c      RRMAX=2.*SX
c      RTMAX=3.*SY
c      RZMAX=2.*SZ
      RRMAX=10.
      IF(ISTORMLEVEL.LE.1)RRMAX=SX
      RTMAX=15.
      RZMAX=1.
      WRITE(6,1517)RRMAX,RTMAX,RZMAX
1517  FORMAT('RADIAL, AZIMUTHAL, AND VERTICAL RADII (KM) OF '/
     + 'INFLUENCE OF DOPPLER DATA WILL BE ',3F8.2)
C
C **** RADIAL, AZIMUTHAL, AND VERTICAL E-FOLDING DISTANCE
C 
      ERMAX=SX
      ETMAX=SY
      EZMAX=SZ
      WRITE(6,1518)ERMAX,ETMAX,EZMAX
1518  FORMAT('RADIAL, AZIMUTHAL, AND VERTICAL E-FOLDING DISTANCE '/
     + 'FOR GAUSSIAN INTERPOLATION WILL BE ',3F8.2,' KM')
C
C **** MAXIMUM ALLOWABLE RADIAL, AZIMUTHAL, AND VERTICAL DISPLACEMENT 
C **** OF WEIGHTED AVERAGE POSITION OF OBSERVATIONS FROM NOMINAL GRID 
C **** POSITION
C 
      XACC=99.
      YACC=99.
      ZACC=99.
C
C **** MAXIMUM AZIMUTHAL DISTANCE (IN DEGREES) THAT AN OBSERVATION 
C **** CAN BE FROM A GRID POINT AND BE INCLUDED IN THE WEIGHTED AVERAGE
C 
      PIMAX=45.
      WRITE(6,1520)PIMAX
1520  FORMAT('MAXIMUM AZIMUTHAL DISTANCE (IN DEGREES) '/
     + ' THAT AN OBSERVATION CAN BE FROM A GRID POINT AND BE '/
     + ' INCLUDED IN THE WEIGHTED AVERAGE WILL BE ',F8.2)
C
C **** AZIMUTHAL COVERAGE FLAG
C 
      IACROSS=1
      IF(IACROSS.EQ.1)THEN
C
C     AZIMUTHS START BEFORE AZI=0 AND EXTEND BEYOND AZI=0, OR FULL 360
C 
       WRITE(6,*)'J=1 AND J=JMAX WILL BE CONSIDERED ADJACENT'
      ELSE
C
C     WEDGE STARTS AT AZI>0 AND ENDS AT AZI<360 
C 
       WRITE(6,*)'J=1 AND J=JMAX WILL NOT BE CONSIDERED ADJACENT ',
     + '(SECTOR ONLY)'
      ENDIF
C
C **** ALLOWS A DIFFERENT NUMBER OF LEVELS BETWEEN SYNTHESIS AND 
C **** ORIGINAL INTERPLOATION (LEGACY)
C 
      KMIN2=1
      KMAX2=KMAX
      WRITE(6,1521)KMIN2,KMAX2
1521  FORMAT('MINIMUM AND MAXIMUM LEVELS OF INTERPOLATION FILE ',
     + 'TO BE FILLED ARE ',2I4)
C
C **** SET MAXIMUM ANGLE OF DOPPLER RADIAL RELATIVE TO HORIZONTAL 
C **** (EARTH-RELATIVE ELEVATION ANGLE) TO BE ACCEPTABLE FOR 
C **** INTERPOLATION (THEN TAKE SIN)
C 
      SINTOL=90.
      WRITE(6,1522)SINTOL
1522  FORMAT('MAXIMUM ANGLE OF DOPPLER RADIAL RELATIVE TO '/
     + 'HORIZONTAL (EARTH-RELATIVE ELEVATION ANGLE) TO BE '/
     + 'ACCEPTABLE FOR INTERPOLATION WILL BE ',F8.2,' DEGREES')
      IF(SINTOL.GE.90.)THEN
       SINTOL=1.
      ELSE
       SINTOL=SINTOL*CTR
       SINTOL=SIN(SINTOL)
      ENDIF
C
C **** MINIMUM DBZ AND RANGE TO INCLUDE
C 
      DBZTEST=-2000.
      RRRR=10.
C
C **** AZIMUTHAL ADVECTION FLAG AND SPEED FOR COMPOSITE
C 
C      IGUESS=0
C 
      IVSPEED=0
      IF(IVSPEED.EQ.1)THEN
       WRITE(6,*)'Enter advection speed'
       READ(1,*)VSPEED0
       VFACTOR=1.
      ELSE
       IF(IGUESS.EQ.5)THEN
        WRITE(6,*)'Enter factor to multiply wind speed by for advection'
        READ(1,*)VFACTOR
       ELSE
        VFACTOR=0.
       ENDIF
       IVSPEED=0
      ENDIF
C
C **** RAMSFILE FLAG AND FILE NAME
C 
      IRAMSFILES=0
      IF(IRAMSFILES.EQ.1)THEN
       DO NF=1,NFILE
        READ(LUIN,'(A80)')RAMSFILES(NF)
        WRITE(6,1530)NF,RAMSFILES(NF)
1530    FORMAT('RAMS FILE FOR UF FILE NO. ',I3,' WILL BE: '/A80)
       ENDDO
      ENDIF
C
C ... INITIALIZE LEGACY VARIABLE
C 
      DO IJK=1,N1
       UGUESS(IJK)=0.
       VGUESS(IJK)=0.
       WGUESS(IJK)=0.
      ENDDO
C
C     ONLY ONE INPUT FILE WHEN WORKING WITH REAL TIME DATA STREAM 
C 
      NF=1
      write(6,*)'at the beginning of nfile do loop nf = ',nf
C
C ... ABCD READS SCAN FILE NO. NF, SUMS EFFECTS FROM PRESENT WIND FILE, 
C     AND COMPUTES WINDS IF LAST FILE IN ITERATION 
C 
C 
      IF(IRAMSFILES.EQ.1)THEN
       CALL OPENRAMFILE(RAMSFILES(NF),LUOUT,STIME,ETIME,DATARATE,IERR)
      ENDIF
C 
      TSTART=TSTARTARRAY(NF)
      TEND=TENDARRAY(NF)
      SLOPE1=DBZFACT(NF)
C 
      IREVERSE=0
      IREADABCD=0
C 
      ISX=0
      WRITE(6,*)'CALLING ABCD and p3radmin = ',P3RADMIN
      WRITE(6,*)'ROLLTOL,IDBZADD,IDBZMASK,NSKIP = ',
     1           ROLLTOL,IDBZADD,IDBZMASK,NSKIP
      WRITE(6,*)'ISTORMLEVEL,IPLANE,IREALTIME,IVARIATIONAL = ',
     1           ISTORMLEVEL,IPLANE,IREALTIME,IVARIATIONAL
      GROUNDADJUST=1.
      WRITE(6,*)'DOPMIN,DOPMAX,DBZMIN,DBZMAX,IZEROWIND = ',
     1           DOPMIN,DOPMAX,DBZMIN,DBZMAX,IZEROWIND
c      PAUSE
      CALL ABCD
     1   (RRMAX,RTMAX,RZMAX,ERMAX,ETMAX,EZMAX,XACC,YACC,ZACC,NF,
     2   SUM1,SUM2,SUM3,SUM4,SUM5,SUM6,SUM7,SUM8,SUM9,
     2   SUM10,SUM11,SUM12,SUM13,SUM14,SUM15,SUM16,SUM17,SUM18,
     2   SUM19,SUM20,SUM21,SUM22,SUM23,SUM24,
     3   KMIN2,KMAX2,
     4   SINTOL,SX,SY,SZ,XZ,YZ,ZZ,ROT,IMAX,JMAX,KMAX,SUMDBZ,
     6   SUMWTSAVE,VELS(1),VELS(1+N1),VELS(1+2*N1),
     6   IGUESS,
     7   IREADABCD,OLAT,OLON,
     9   SLOPE1,CENTIME,TSTART,TEND,
     1   SMOTIONU,SMOTIONV,FLTNAME,DBZTEST,RRRR,
     1   UGUESS,VGUESS,WGUESS,IREVERSE,IRAMSFILES,
     1   DISTXYZ,PIMAX,IACROSS,IVSPEED,VSPEED0,VFACTOR,TATF,IGRAPHICS,
     1   ZLOW,ZHIGH,HB2,DPB2,ISTORMLEVEL,IPLANE,IREALTIME,IVARIATIONAL,
     1   ISX,IAFTFORE,ISIGMET,ROLLTOL,NSKIP,IDBZADD,IDBZMASK,P3RADMIN,
     1   DOPMIN,DOPMAX,DBZMIN,DBZMAX,IZEROWIND,IKLUGETEST,IREMOVEGROUND,
     1   IUNFLD,IAZIUNFLD,IDESPECKLE,RADRAT,RADMASK,IFILLGAP,
     1   SHORTPULSEADDDB,SHORTPULSEADDRADIUS)
C
C ... CLOSE FILE OPENED IN SUBROUTINE ABCD
C 
      CLOSE(98)
C 
      IWRITEABCD=1 
C 
      IF(IWRITEABCD.EQ.1)THEN
C
C      LUW IS THE LOGICAL UNIT NUMBER FOR OUTPUT SUMFILE. FILE IS OPENED
C      IN MAIN PROGRAM BUT CLOSED IN A SUBROUTINE
C 
       LUW=95
       NFF=1
       IF(IAFTFORE.EQ.1.OR.IAFTFORE.EQ.3)THEN
        OPEN(LUW,FILE=OSUMFILENAME(NFF),FORM='UNFORMATTED')
C
C .... WRITEABCD WRITES OUT THE SUMMATIONS OF PRODUCTS THAT ARE NEEDED FOR
C      THE LEAST SQUARES WIND SOLUTION IN THE SYNTHESIS PROGRAMS
C 
        CALL WRITEABCD24(LUW,IMAX,JMAX,KMAX,SUM1,SUM2,SUM3,
     +   SUM4,SUM5,SUM6,SUM7,SUM8,SUM9,SUM10,SUM11,SUM12,SUM13,
     +   SUM14,SUM15,SUM16,SUM17,SUM18,SUM19,SUM20,SUM21,SUM22,
     +   SUM23,SUM24,SUMDBZ,
     +   VELS(1),VELS(N1+1),VELS(2*N1+1),NFF)
        write(6,*)'ended writeabcd for file ',nf
C
C      APPARENTLY FILE LUW IS CLOSED IN WRITEABCD
C 
       ENDIF
       IF(IAFTFORE.EQ.2.OR.IAFTFORE.EQ.3)THEN
        NFF=2
        OPEN(LUW,FILE=OSUMFILENAME(NFF),FORM='UNFORMATTED')
        CALL WRITEABCD24(LUW,IMAX,JMAX,KMAX,SUM1,SUM2,SUM3,
     +   SUM4,SUM5,SUM6,SUM7,SUM8,SUM9,SUM10,SUM11,SUM12,SUM13,
     +   SUM14,SUM15,SUM16,SUM17,SUM18,SUM19,SUM20,SUM21,SUM22,
     +   SUM23,SUM24,SUMDBZ,
     +   VELS(1),VELS(N1+1),VELS(2*N1+1),NFF)
        write(6,*)'ended writeabcd for file ',nf
       ENDIF
      ENDIF
      STOP
      END
