      PROGRAM JOBFILE_EDIT
C Program designed to modify jobfile_radar_profile, jobfile_radar_profile_in
C and jobfile_radar_profile_out so they extend as far as the wavenumber 0/1 
C analysis.  Generally they do not extend all the way out to maximum radius
C
C If radial resolution of wavenumber 0/1 greater than or equal to 
C 1.5 km, the profile radial resolution is set to 1.5 km.  
C
C If radial resolution of wavenumber 0/1 less than 1.5 km, then profiles
C have the same radial resolution oas the wavenumber 0/1 analysis
C
      CHARACTER*8 FLIGHTID
      CHARACTER*80 DIRECTORY
      CHARACTER*12 STORMNAME
      CHARACTER*30 OUTFILE_JOBFILE_RADAR
      CHARACTER*28 OUTFILE_JOBFILE_RADAR_IN
      CHARACTER*29 OUTFILE_JOBFILE_RADAR_OUT
      INTEGER TIMEXSBEGIN,TIMEXSEND
      INTEGER TIME3DBEGIN,TIME3DEND,TIMECENTER
C 'jobfile_radar' has the start and end time of full 3D analysis, as well
C as other navigation and timing information
      OPEN(81,FILE='jobfile_radar')
C
C 'jobfile_radar_in has the begin and end times for inbound leg'
      OPEN(82,FILE='jobfile_radar_in')
C
C 'jobfile_radar_out has the begin and end times for outbound leg'
      OPEN(83,FILE='jobfile_radar_out')
C
C 'jobfile_radar_rt has the radial resolution of wavenumber 0/1 analysis'
      OPEN(84,FILE='jobfile_radar_rt')
C
      READ(81,'(A)')FLIGHTID
      READ(82,'(A)')FLIGHTID
      READ(83,'(A)')FLIGHTID
      READ(84,'(A)')FLIGHTID
      READ(81,'(A)')STORMNAME
      READ(82,'(A)')STORMNAME
      READ(83,'(A)')STORMNAME
      READ(84,'(A)')STORMNAME
C
C Read full time span of 3D analysis
      READ(81,*)TIME3DBEGIN,TIME3DEND
C
C Read full time span of inbound profile analysis
      READ(82,*)TIMEXSBEGIN,TIMECENTER
C
C Read full time span of outbound profile analysis
      READ(83,*)TIMECENTER,TIMEXSEND
C 
      READ(84,*)TIME3DBEGIN,TIME3DEND
C
C Read West-East, South-North components of storm motion
      READ(81,*)U,V
      READ(82,*)U,V
      READ(83,*)U,V
      READ(84,*)U,V
C
C Time the center of the circulation was fixed, or best estimate of time
C storm center would have been fixed
      READ(81,*)TIMECENTER
      READ(82,*)TIMECENTER
      READ(83,*)TIMECENTER
      READ(84,*)TIMECENTER
C
C Best estimate of storm center latitude and longtitude of time of TIMECENTER
C Estimate may come from a fix, and generally be better
      READ(81,*)RLAT,RLON
      READ(82,*)RLAT,RLON
      READ(83,*)RLAT,RLON
      READ(84,*)RLAT,RLON
C
C Read zonal, meridional, and vertical resolution of 3D analysis in km
      READ(81,*)SX,SY,SZ
C
C Read location of center relative to SW corner of 3D analysis (XZ,YZ),
C altitude of lowest level of 3D analysis (ZZ), and rotation of y-axis 
C from being due north (ROT)
      READ(81,*)XZ,YZ,ZZ,ROT
C IMAX,JMAX,KMAX integral domains in E-W, N-S, and vertical dimensions
C The data read before next comment are the same in: jobfile_radar,
C jobfile_radar_in, and jobfile_radar_out
      READ(81,*)IMAX,JMAX,KMAX
      READ(82,*)SX,SY,SZ
      READ(82,*)XZ,YZ,ZZ,ROT
      READ(82,*)IMAX,JMAX,KMAX
      READ(83,*)SX,SY,SZ
      READ(83,*)XZ,YZ,ZZ,ROT
      READ(83,*)IMAX,JMAX,KMAX
C
C Read radial (SR-degrees), azimuthal (ST-degrees), and lowest altitude (SZ-km)
      READ(84,*)SR,ST,SZ
C
C Read location of first bin of wavenumber 0/1 analysis in radius (RZ-km), 
C aziumth (TZ-degrees), and lowest level of analysis (ZZ-km)
      READ(84,*)RZ,TZ,ZZ
C
C Integer dimension of analysis in radius (MAXR), azimuth (MAXT) and 
C altitude (MAXZ)
      READ(84,*)MAXR,MAXT,MAXZ
C
C Radiusmax is the full extent of wavenumber 0/1 analysis
      RADIUSMAX=SR*MAXR
C
C Set the radial resolution and radial integer dimension
      IF(SR.GT.1.5)THEN
C 
C Set profile radial resolution for wavenumber 0/1 analyses that have a radial 
C resolution of 1.5 km or greater
         SRPROFILE=1.5
         MAXRPROFILE=NINT(RADIUSMAX/SRPROFILE)
         RZPROFILE=SRPROFILE
      ELSE
C
C Set profile radial resolution for wavenumber 0/1 analyses that have a radial
C resolution less than 1.5 km
         SRPROFILE=SR
         MAXRPROFILE=MAXR
         RZPROFILE=RZ
      ENDIF
C
C From heere to next comment data are read and then repeated in 
C profile jobfiles.  Beginning of copy...
      READ(81,*)MINUS1
      READ(82,*)MINUS1
      READ(83,*)MINUS1
      READ(84,*)MINUS1
C ISTARTGATE how many radial bins out from aircraft to be ignored
C INOB Number of radial-velocity observations to be averaged before comparing 
C to next radial bin when running Bargen-Brown de-aliasing
      READ(81,*)ISTARTGATE,INOB
      READ(82,*)ISTARTGATE,INOB
      READ(83,*)ISTARTGATE,INOB
      READ(84,*)ISTARTGATE,INOB
C Reading the angle-correction values for aft antenna from all four input files
      READ(81,*)AFT1,AFT2,AFT3
      READ(82,*)AFT1,AFT2,AFT3
      READ(83,*)AFT1,AFT2,AFT3
      READ(84,*)AFT1,AFT2,AFT3
      READ(81,*)AFT4,AFT5,AFT6
      READ(82,*)AFT4,AFT5,AFT6
      READ(83,*)AFT4,AFT5,AFT6
      READ(84,*)AFT4,AFT5,AFT6
C The same for the fore antenna
      READ(81,*)FORE1,FORE2,FORE3
      READ(82,*)FORE1,FORE2,FORE3
      READ(83,*)FORE1,FORE2,FORE3
      READ(84,*)FORE1,FORE2,FORE3
      READ(81,*)FORE4,FORE5,FORE6
      READ(82,*)FORE4,FORE5,FORE6
      READ(83,*)FORE4,FORE5,FORE6
      READ(84,*)FORE4,FORE5,FORE6
C Other processing parameters that will be the same in all input and output
C jobfiles
      READ(81,*)A1,A2,A3,A4
      READ(82,*)A1,A2,A3,A4
      READ(83,*)A1,A2,A3,A4
      READ(84,*)A1,A2,A3,A4
C THRESH is a signal-quality nuber that will be the same in input and output
C job files
      READ(81,*)THRESH
      READ(82,*)THRESH
      READ(83,*)THRESH
      READ(84,*)THRESH
C Beginning and ending time of 3D analysis and RT repeated in the file
      READ(81,*)ITIMEDUMMY1,ITIMEDUMMY2
      READ(82,*)ITIMEDUMMY1,ITIMEDUMMY2
      READ(83,*)ITIMEDUMMY1,ITIMEDUMMY2
      READ(84,*)ITIMEDUMMY1,ITIMEDUMMY2
C Read continuity FACTOR, which is the amount to multiply the continuity-
C equation part of the cost function to get a good continuity solution
      READ(81,*)FACTOR
      READ(82,*)FACTOR
      READ(83,*)FACTOR
      READ(84,*)FACTOR
C HTMELT is the height in km halfway through the melting band
C TKMELT is the vertical thickness of the melting band in km
      READ(81,*)HTMELT,TKMELT
      READ(82,*)HTMELT,TKMELT
      READ(83,*)HTMELT,TKMELT
      READ(84,*)HTMELT,TKMELT
C BOTDB is actually ZLOW (look in jobfiletoparameters.f to see description)
C TOPDB is actually ZLOW (look in jobfiletoparameters.f to see description)
      READ(81,*)BOTDB,TOPDB
      READ(82,*)BOTDB,TOPDB
      READ(83,*)BOTDB,TOPDB
      READ(84,*)BOTDB,TOPDB
C Names of the HRD-style wind files that would be the output from analysis
      READ(81,'(A)')OUTFILE_JOBFILE_RADAR
      READ(82,'(A)')OUTFILE_JOBFILE_RADAR_IN
      READ(83,'(A)')OUTFILE_JOBFILE_RADAR_OUT
      READ(84,'(A)')OUTFILE_JOBFILE_RADAR_RT
C ANGLEINBOUND Nominal(or average) heading from storm center of inbound profile
C ANGLEOUTBOUND Nominal(or average) heading from storm center outbound profile
      READ(81,*)ANGLEINBOUND,ANGLEOUTBOUND
      READ(82,*)ANGLEINBOUND,ANGLEOUTBOUND
      READ(83,*)ANGLEINBOUND,ANGLEOUTBOUND
C Directory holding the Tail Doppler radar file (rvp5) or sweeps (rvp8+)
      READ(81,*)DIRECTORY
      READ(82,*)DIRECTORY
      READ(83,*)DIRECTORY
C IANT antenna number on aircraft
C 0 noaa parabolic antenna for P3
C 1 French antenna for P3
C 2 G-IV antenna
C 3 Malibu-built antenna for P3 (introduced on P3 during the 2010's)
      READ(81,*)IANT
      READ(82,*)IANT
      READ(83,*)IANT
C ISTORM ATCD storm number 
C 0-99 Generic--generally not used
C 101-199 for Atlantic
C 201-299 for East Pacific
C 301-399 for Central Pacific
      READ(84,*)ISTORM
      READ(81,*)ISTORM
      READ(82,*)ISTORM
      READ(83,*)ISTORM
      CLOSE(81)
      CLOSE(82)
      CLOSE(83)
      CLOSE(84)
C
C End of copy
C
C Open output files for setting up profile analyses that match the
C resolution and dimension of wavenumber 0/1 analyses
C jobfile_radar_profile a generic profile not actually used in analyses
      OPEN(82,FILE='jobfile_radar_profile')
C jobfile_radar_profile_in has the proper dimensions for the inbound profile
C x-section
      OPEN(83,FILE='jobfile_radar_profile_in')
C jobfile_radar_profile_in has the proper dimensions for the outbound profile
C x-section
      OPEN(84,FILE='jobfile_radar_profile_out')
      WRITE(82,'(A8)')FLIGHTID
      WRITE(83,'(A8)')FLIGHTID
      WRITE(84,'(A8)')FLIGHTID
      WRITE(82,'(A12)')STORMNAME
      WRITE(83,'(A12)')STORMNAME
      WRITE(84,'(A12)')STORMNAME
C jobfile_radar_profile  for generic profile only and uses 3D times
      WRITE(82,'(I6,X,I6)')TIMEXSBEGIN,TIMEXSEND
C jobfile_radar_profile_in if for inbound profile only and uses those times
      WRITE(83,'(I6,X,I6)')TIMEXSBEGIN,TIMECENTER
C jobfile_radar_profile_out if for outbound profile only and uses those times
      WRITE(84,'(I6,X,I6)')TIMECENTER,TIMEXSEND
      WRITE(82,'(2F8.2)')U,V
      WRITE(83,'(2F8.2)')U,V
      WRITE(84,'(2F8.2)')U,V
      WRITE(82,'(I6)')TIMECENTER
      WRITE(83,'(I6)')TIMECENTER
      WRITE(84,'(I6)')TIMECENTER
      WRITE(82,'(2F10.3)')RLAT,RLON
      WRITE(83,'(2F10.3)')RLAT,RLON
      WRITE(84,'(2F10.3)')RLAT,RLON
C
C Profile vertical resolution still hardwired to 0.15 km
      SZP=0.15
      WRITE(82,'(3F8.2)')SRPROFILE,ST,SZP
      WRITE(83,'(3F8.2)')SRPROFILE,ST,SZP
      WRITE(84,'(3F8.2)')SRPROFILE,ST,SZP
      WRITE(82,'(4F8.2)')RZPROFILE,TZ,ZZ,ROT
      WRITE(83,'(4F8.2)')RZPROFILE,TZ,ZZ,ROT
      WRITE(84,'(4F8.2)')RZPROFILE,TZ,ZZ,ROT
C
C Profile vertical integral dimension still hardwired to 121, so 
C profiles are hardwired to a vertical resolution of .15 km
C and a vertical domain of 18 km.
      KMAX121=121
      WRITE(82,'(3I5)')MAXRPROFILE,MAXT,KMAX121
      WRITE(83,'(3I5)')MAXRPROFILE,MAXT,KMAX121
      WRITE(84,'(3I5)')MAXRPROFILE,MAXT,KMAX121
      WRITE(82,'(I3)')MINUS1
      WRITE(83,'(I3)')MINUS1
      WRITE(84,'(I3)')MINUS1
      WRITE(82,'(2I4)')ISTARTGATE,INOB
      WRITE(83,'(2I4)')ISTARTGATE,INOB
      WRITE(84,'(2I4)')ISTARTGATE,INOB
      WRITE(82,'(4F8.2)')AFT1,AFT2,AFT3
      WRITE(83,'(4F8.2)')AFT1,AFT2,AFT3
      WRITE(84,'(4F8.2)')AFT1,AFT2,AFT3
      WRITE(82,'(4F8.2)')AFT4,AFT5,AFT6
      WRITE(83,'(4F8.2)')AFT4,AFT5,AFT6
      WRITE(84,'(4F8.2)')AFT4,AFT5,AFT6
      WRITE(82,'(4F8.2)')FORE1,FORE2,FORE3
      WRITE(83,'(4F8.2)')FORE1,FORE2,FORE3
      WRITE(84,'(4F8.2)')FORE1,FORE2,FORE3
      WRITE(82,'(4F8.2)')FORE4,FORE5,FORE6
      WRITE(83,'(4F8.2)')FORE4,FORE5,FORE6
      WRITE(84,'(4F8.2)')FORE4,FORE5,FORE6
      WRITE(82,'(4F8.2)')A1,A2,A3,A4
      WRITE(83,'(4F8.2)')A1,A2,A3,A4
      WRITE(84,'(4F8.2)')A1,A2,A3,A4
      WRITE(82,'(F5.2)')THRESH
      WRITE(83,'(F5.2)')THRESH
      WRITE(84,'(F5.2)')THRESH
      WRITE(82,'(I6,X,I6)')TIMEXSBEGIN,TIMEXSEND
      WRITE(83,'(I6,X,I6)')TIMEXSBEGIN,TIMECENTER
      WRITE(84,'(I6,X,I6)')TIMECENTER,TIMEXSEND
      WRITE(82,'(F10.1)')FACTOR
      WRITE(83,'(F10.1)')FACTOR
      WRITE(84,'(F10.1)')FACTOR
      WRITE(82,'(2F10.2)')HTMELT,TKMELT
      WRITE(83,'(2F10.2)')HTMELT,TKMELT
      WRITE(84,'(2F10.2)')HTMELT,TKMELT
      WRITE(82,'(2F10.2)')BOTDB,TOPDB
      WRITE(83,'(2F10.2)')BOTDB,TOPDB
      WRITE(84,'(2F10.2)')BOTDB,TOPDB
C
C Append Flight ID and center time info before file-name suffix for
C profile files
      WRITE(82,'(A13,"_vert.w")')OUTFILE_JOBFILE_RADAR
      WRITE(83,'(A13,"_vert_inbound.w")')OUTFILE_JOBFILE_RADAR
      WRITE(84,'(A13,"_vert_outbound.w")')OUTFILE_JOBFILE_RADAR
      WRITE(82,'(I3)')ISTORM
      WRITE(83,'(I3)')ISTORM
      WRITE(84,'(I4)')ISTORM
      WRITE(82,'(2F10.2)')ANGLEINBOUND,ANGLEOUTBOUND
      WRITE(83,'(2F10.2)')ANGLEINBOUND,ANGLEOUTBOUND
      WRITE(84,'(2F10.2)')ANGLEINBOUND,ANGLEOUTBOUND
      CLOSE(82)
      CLOSE(83)
      CLOSE(84)
      STOP
      END








