      SUBROUTINE RAYCOR1(bin, zbin)
C     THIS SUBROUTINE CORRECTS THE FLIGHT DATA ONE RAY AT A TIME THEN PASSES
C     IT BACK FOR AVERAGING, INTERPOLATION, OR FURTHER ANALYSIS.
C     THIS ROUTINE WORKS IN THE AIRCRAFT COORDINATE SYSTEM OF ANGLES.
C     AZM    0 - 360 DEGREES  SWEEP ROTATION ANGLE(CLOCKWISE) WITH 0 UP
C     ELEV  (+OR-)25 DEGREES  ANTENNA ANGLE(TILT) FROM PERPENDICULAR TO TRACK
C  	                      >0 IS FORWARD, <0 IS AFT, 0 IS PERPENDICULAR
C     TRACK  0 - 360 DEGREES  ANGLE AIRCRAFT TRACK MAKES FROM NORTH(0 DEGREES)
C                             METEOROLOGICAL ANGLES
C     BIN(512) = ARRAY CONTAINING THE DOPPLER DATA TO BE RETURNED
C     ZBIN(512) = ARRAY CONTAINING THE REFLECTIVITY(DBZ) DATA TO BE RETURNED
C fragments for reading HRD tape or file
C note change in array dims from 512 to 1024 for rvp8
c	parameter(bad_flag=-900.0, missing = -9999)
c	parameter( bad_w = -9000.0)
c	parameter(max_hrd_dim = 1024)
      include "hrd_radar_settings.f"
      DIMENSION      bin(max_hrd_dim), zbin(max_hrd_dim)
      DIMENSION      bin2(max_hrd_dim)
C
C Radar tape/file stuff
	COMMON /TAPEHEADER/ 
     +	nbytesheader,ntape,nversion,ihheader,imheader,
     +  isheader,ine,ndrive,naircraft,flightid,stmname,
     +  nsamplelf,nsampleta,refslopelf,refslopeta,
     +  refnoisethreslf,refnoisethresta,sqilf,sqita,
     +  widththreslf,widththresta,caliblf,calibta,
     +  modedatalf,modedatata,noisetmlf,noisetmta,
     +  rpmlf,rpmta,gtlnlf,ivrangelf,ivrangeta,
     +   gtlnta(3),rdellf,rdelta,rdelcorlf,rdelcorta,
     +   nbinlf,nbinta,nbins,rmaxlf,rmaxta,
     +   radiita(max_hrd_dim),radiilf(max_hrd_dim),
C following 6 variables are new:
     +	 prf_flaglf, prf_flagta, prf_highlf,
     +	 prf_highta, xniqlf, xniqta, 
     +	iheader(1024)
C
	COMMON /RAYHEADER / 
     + nbytesdata,nlfsweep,ntasweep,numrecord,
     + irecflag,idradar,nbytesray,icode,idsp,mdata,
     + iyear,month,iday,iraycode,radtime,ih,im,
     + is,is100,radlat,radlon,radalt,gse,gsn,radgs,radvgs,
     + ucom,vcom,radwd,radws,radvws,ircu,
     +  elev,radazm,pitch,roll,drift,heading,
     +  dbzbuffer(max_hrd_dim),
     +	velbuffer(max_hrd_dim),
     +   widthbuffer(max_hrd_dim)
C
	integer*2 iheader
	integer prf_flaglf, prf_flagta
      INTEGER LUT,BIN1,BIN2,NWEDGE,KINDRADAR, time_ta, time_lf
	integer bins_lf, bine_lf, bins_ta, binm_ta, bine_ta
      CHARACTER FLIGHTID*8,STMNAME*16
      COMMON /PARAMS/trackazm, trackelev, driftnew,
     +               rot, track, htbb, dbb, dbzthresh, swthresh, dbzrs,
     +               ibst(2), inob(2), ileg, xniq, irsw, maxnbins, v0,
     +               pcor, dcor, rcor, azmcor, elcor, dbzcor, dbzslope,
     +               iunfoldflag, iattenflag,
     +               badstabflag, surfelimflag, echoflag, atmflag
      LOGICAL        badstabflag, surfelimflag, echoflag, atmflag,
     +               swflag
      REAL           amos(max_hrd_dim)
      PARAMETER      (cos_bw = 0.999999958, sin_bw = 0.0002893864959)
      DATA           pidiv180/0.01745329/, amos/max_hrd_dim*0.0/,
     +               swflag/.FALSE./

C     CALCULATE FUNCTIONS USED REPEATEDLY
c      WRITE(6,5656)BIN
 5656 FORMAT(10F8.1)
c      WRITE(6,*)'MAX_HRD_DIM = ',MAX_HRD_DIM
c      WRITE(6,*)'RAYCOR PAUSE'
c      PAUSE
c      write(6,*)'entering corvl = ',corvl
      caz = COS(trackazm * pidiv180) 
      saz = SIN(trackazm * pidiv180) 
      cev = COS(trackelev * pidiv180)
      sev = SIN(trackelev * pidiv180)
      cazcev = caz * cev
      sazcev = saz * cev

c      write(6,*)'raycor 1'
C     PROCESS DOPPLER:
      nb = nbins
      nbd = nbins
c      write(6,*)'nb,nbd,nbins = ',nb,nbd,nbins
C     IGNORE DATA BELOW THE GROUND: CALCULATE THE BIN VALUE AT THE GROUND
C     ASSUME HALF BEAMWIDTH OF 0.95 DEG(USES COS AND SIN OF 0.95)
      IF (caz.LT.0.0) THEN
        cs = (caz * cos_bw + SIGN(1.0, saz) * saz * sin_bw) * cev
        a = -radalt / cs
        b = rdelta * 1000.0
        c = gtlnta(1) * 1000.0
        nbd = NINT((a - b) / c)
        IF ((ivrangeta.EQ.0).OR.(nbd.LE.256)) THEN
          CONTINUE
        ELSE IF (nbd.LE.512) THEN
          nbd = (nbd - 255) / 2 + 256
        ELSE 
          nbd = (nbd - 509) / 4 + 384
        ENDIF

C       SUBTRACT 1 FROM THE NUMBER OF BINS TO UNFOLD IN CASE INOB=0,
C       YOU DON'T WANT TO USE THE INTERSECTION BIN 
        nb = nbd - ibst(ileg) - 1
      ENDIF

C     USE THE MINIMUM NUMBER OF BINS REQUESTED OR CALCULATED
c      write(6,*)'nb = ',nb
c      write(6,*)'maxnbins = ',maxnbins
      nb = MIN0(maxnbins, nb)
c      write(6,*)'nb = ',nb


C     CALCULATE BIN# AT RADALT TO REMOVE THE SIDELOBES.  THE ECHO MUST BE
C     > ZBRA(RANGE CORRECTION IN THE DBZ COMPUTATION) FOR THE GATES TO BE KEPT
      nbra = NINT((radalt * 0.001 - rdelta) / gtlnta(1)) + 1
      zbra = 20.0 * ALOG10(radalt * 0.001) + 1.0

C     COMPUTE ATMOSPHERIC ATTENUATION
C      IF (atmflag) THEN
C        CALL ATM(cazcev, sazcev, idradar, radalt, radiita, nbins,amos)
C      ENDIF

C     CORRECTION OF DOPPLER VELOCITIES FOR IMPROPER ANTENNA BEHAVIOR(STABILITY).
C     GIVEN SPEED ALONG TRACK(RADGS), VERTICAL MOTION OF AIRCRAFT(RADVGS), CORRECTION
C     OF VELOCITY IS A PROJECTION OF THE AIRCRAFT MOTION VECTOR ON THE RADAR RAY
c      write(6,*)'raycor 1a'
      IF (badstabflag) THEN
        corvl = radgs * sev + radvgs * cazcev
      ELSE
        corvl = 0.0
      ENDIF
c      write(6,*)'raycor 2a'
C     LOAD DBZ VALUES INTO ARRAY ZBIN ADDING APPLICABLE CORRECTIONS
c      write(6,*)'raycor 2'
c      write(6,*)'nbins = ',nbins
      DO i = 1, nbins
        IF (dbzbuffer(i).GE.-31.5) THEN
          zbin(i) = dbzcor + dbzslope * dbzbuffer(i) + amos(i)
        ELSE
          zbin(i) = dbzbuffer(i)
        ENDIF
      ENDDO
c      write(6,*)'raycor 3'
C     ADD INTERVENING ATTENUATION IF DESIRED
  
      IF (iattenflag.EQ.1) THEN
       CALL INTERVENATTEN1(zbin, nbins, radiita)
c       WRITE(6,*)'CALLED INTERVENATTEN1'
      ENDIF

C     DECIDE IF SPECTRAL WIDTH THRESHHOLDING IS BEING DONE
c      write(6,*)'raycor 4'
      IF ((swthresh.LE.12.75).AND.(swthresh.GT.0.0).AND.
     +  (ibits(icode, 13, 1).EQ.1)) swflag = .TRUE.
       IBITSY=IBITS(ICODE,13,1)
c       WRITE(6,*)'SWFLAG,IBITSY = ',SWFLAG,IBITSY
c      write(6,*)'raycor 5'

C     CALCULATE RANGE DEPENDENT DBZ THRESHOLD
c      write(6,*)'nbins = ',nbins
c      write(6,*)'velbuffer = ',velbuffer
      DO i = 1, nbins
c         if(velbuffer(i).gt.-800..and.trackelev.gt.0.)then
c            bin(i)=-velbuffer(i)
c         else
            bin(i)=velbuffer(i)
c         endif
        dbzt = dbzthresh
c        if(abs(bin(i)).gt.13..and.abs(bin(i)).lt.100.)then
c         write(6,*)'i,bin(i),velbuffer(i) = ',i,bin(i),velbuffer(i)
c        endif
c        write(6,*)'raycor 1aa'
        IF (dbzrs.GT.0) THEN
          rdb = radiita(i)
          IF (rdb.GE.dbzrs)dbzt = dbzthresh + 20.0 * ALOG10(rdb / dbzrs)
        ENDIF
c        write(6,*)'raycor 1ab'
        zbra = AMAX1(zbra, dbzt)
c        write(6,*)'raycor 1ac'
C       IGNORE ALL UNKNOWN VELOCITIES 
        IF (bin(i).LE.-888.0) THEN
         IF(WIDTHBUFFER(I).LT.0.)ZBIN(I)=-32.
c          write(6,*)'raycor 1ba'
          CONTINUE  ! GO TO ENDDO

C       IGNORE VALUES WHERE RANGE DELAY CORRECTION HAS MADE A NEGATIVE RADII
        ELSE IF (radiita(i).LT.0.0) THEN  ! PAL 5 FEB 93
          bin(i) = -888.88
          zbin(i) = -32.0
c          write(6,*)'raycor 1bb'
C       IGNORE ALL VELOCITIES BELOW THE SEA SURFACE
        ELSE IF ((i.GT.(MIN0(maxnbins, nbd))).AND.(surfelimflag)) THEN
          bin(i) = -888.88
          zbin(i) = -32.0
c          write(6,*)'raycor 1bc'
C       THRESHOLD VALUES BASED ON SPECTRAL WIDTH IF DESIRED - PAL 5 OCT 94
        ELSE IF (((swflag).AND.
ccc     +    (widthbuffer(i).GT.swthresh).AND.(zbin(i).LT.dbzt)) THEN
     +    (widthbuffer(i).GT.swthresh).OR.WIDTHBUFFER(I).LT.0.)) THEN  ! Edited 18 Feb 97 per FM
c          write(6,*)'i,widthbuffer,bin,zbin = ',
c     +     i,widthbuffer(i),bin(i),zbin(i)
          bin(i) = -888.88
          zbin(i) = -32.0
c          write(6,*)'raycor 1bd'

C       IGNORE ALL VELOCITIES AT ECHO(RANGE) RING AROUND AIRPLANE
        ELSE IF (((.NOT.(swflag)).AND.
     +    ((i - nbra).LT.7).AND.((i - nbra).GT.-7)).AND.
     +    ((zbin(i).LT.zbra).AND.(echoflag)))  THEN 
          bin(i) = -888.88 
          zbin(i) = -32.0
c          write(6,*)'raycor 1be'
C       OR USE VARIABLE RANGE DBZ THRESHOLDING 
ccc        ELSE IF ((.NOT.(swflag)).AND.(zbin(i).LT.dbzt)) THEN 
        ELSE IF (zbin(i).LT.dbzt) THEN   ! Edited 18 Feb 97 per FM
          bin(i) = -888.88 
          zbin(i) = -32.0  ! Edited 18 Feb 97 per FM
c          write(6,*)'raycor 1bf'
C       OTHERWISE UNPACK VELOCITY AND CORRECT FOR ANTENNA ATTITUDE AND FALLSPEED
        ELSE
c         write(6,*)'swthresh,widthbuffer(i) = ',swthresh,widthbuffer(i)  
c          write(6,*)'raycor 1ca'
          IF (htbb.LT.0.0) THEN
            corvt = 0.0
c             write(6,*)'raycor 1cb'
          ELSE 

C           CORRECTION OF DOPPLER VELOCITY ESTIMATES FOR PARTICLE FALL SPEED(VT)
C           USING DBZ-VT RELATIONSHIP TO ESTIMATE VT.  IF IRSW = 0 THEN FUNCTION
C           USES THE JOSS + WALDVOGEL METHOD.  GIVEN A PARTICLE FALL SPEED(VT),
C           DERIVED FROM REFLECTIVITY OR CLOUD MICROPHYSICS, THE CORRECTION TO
C           THE RADIAL VELOCITY IS THE PROJECTION OF THE VERTICAL VELOCITY ON
C           THE RADAR RAY.
c            hgt = HEIGHTBEAM(radiita(i), radalt*0.001, cazcev, sazcev)
c            corvt = - VT(zbin(i), hgt, htbb, dbb, irsw) * cazcev
           CORVT=0.
          ENDIF
c          write(6,*)'before i,bin = ',i,bin(i)
c          bin(i) = -bin(i) + corvl + corvt 
          bin(i) = bin(i) + corvl + corvt 
c          bin(i) = -bin(i)
c          write(6,*)'after i,corvl,corvt, bin = ',i,corvl,corvt,bin(i)
          DO WHILE(ABS(BIN(I)).GT.XNIQTA)
c           write(6,*)'i,bin(i),xniq = ',i,bin(i),xniqta
           IF (bin(i).LT.-xniq) bin(i) = bin(i) + 2.0 * xniq
           IF (bin(i).GT.xniq) bin(i) = bin(i) - 2.0 * xniq 
c           write(6,*)'i,bin(i),xniq = ',i,bin(i),xniq
c           pause
          ENDDO
c          write(6,*)'raycor 1db'
        ENDIF
      ENDDO
C     TRACK ANGLE IS THE HEADING PLUS THE CORRECTED DRIFT
c      pause
c      write(6,*)'raycor,heading,drift,driftnew = ',
c     1           heading,drift,driftnew
      track = AMOD(heading + driftnew + 360.0, 360.0)
c      write(6,*)'raycor track = ',track
C     THE SEED VELOCITY FOR DOPPLER UNFOLDING IS DERIVED FROM WIND SPEED AT 
C     FLIGHT LEVEL.  GIVEN TRACK, HORIZONTAL WIND SPEED(RADWS), WIND DIRECTION(RADWD),
C     WHERE RADWD IS THE DIRECTION WIND IS BLOWING FROM, AND THE VERTICAL WIND
C     SPEED(VRADWS), WE CAN DEFINE A VELOCITY VECTOR RELATIVE TO TRACK.
        wd_prime = (radwd - track) * pidiv180  ! WIND DIRECTION RELATIVE TO TRACK
        v0 = -radws * (sazcev * SIN(wd_prime) + sev*COS(wd_prime))
     +    + radvws * cazcev
c        write(6,*)'wd,wd_prime,v0,ws,radvws = ',wd,wd_prime,v0,ws,radvws
      IF (iunfoldflag.EQ.1) THEN
        IF (nb.LT.1) RETURN
        wd_prime = (radwd - track) * pidiv180  ! WIND DIRECTION RELATIVE TO TRACK
        v0 = -radws * (sazcev * SIN(wd_prime) + sev * COS(wd_prime)) +
     +    radvws * cazcev
c        write(6,*)'v0 = ',v0
c      write(6,*)'before unfld bin = ',bin
      DO I=1,max_hrd_dim
       BIN2(I)=-888.88
      ENDDO
        CALL UNFLD1(v0, xniq, inob(ileg), ibst(ileg), nb, bin(1),
     +    bin2(1), nbins)
c      write(6,*)'after unfld bin = ',bin2
        DO I=1,max_hrd_dim
         BIN(I)=BIN2(I)
        ENDDO
      ENDIF 
c      WRITE(6,5656)BIN
c      WRITE(6,*)'LEAVING RAYCOR PAUSE'
c      PAUSE
c      write(6,*)'returning corvl,corvlist,xniq,xniqta = ',
c     + corvl,corvlist,xniq,xniqta
      RETURN
      END
