      SUBROUTINE READWIND(U,V,W,IMX,JMX,KMX,SX,SY,SZ,
     1 OLAT,OLON,XZ,YZ,ZZ,IJKMAX,ZLOW,ZHIGH,HB2,DPB2)
C 
C     GUESS WIND FIELD SUBROUTINE CALLED BY ABCD (ABCD_SCAN_RT4.F)
C       
C ... FUNCTION:   
C       
C     THIS SUBROUTINE READS IN A CYLINDRICAL GUESS WIND FIELD
C
C ... READWIND SUBPROGRAM REFERENCES:      
C       
C          NON-GRAPHICS     READHEADERS, LDPLN
C
C     VERSION:  <TBD>
C       
C     CODE WRITTEN BY:  JOHN GAMACHE   
C                       HRD/AOML/NOAA          
C                       MIAMI, FL 33149      
C       
C     FOR ADDITIONAL DETAILS ON THE ALGORITHM SEE <TBD>
C       
C     BASED ON THEORY BY:  <TBD>
C       
C     REFERENCE THE BOOK:  <TBD>
C       
C     **************************************************  
C     *               IMPORTANT NOTE                   *  
C     *                                                *  
C     *                    <TBD>                       *  
C     *                                                *  
C     **************************************************         
C
C     SPECIFICATIONS FOR ARGUMENTS
C 
      INTEGER IMX,JMX,KMX
      REAL U(IJKMAX),V(IJKMAX),W(IJKMAX)
      REAL OLAT,OLON,SX,SY,SZ,XZ,YZ,ZZ
      REAL ZLOW,ZHIGH,HB2,DPB2
C 
C ... DESCRIPTION OF ARGUMENTS
C
C     IMX,JMX,KMX      - GUESS WIND RADIAL,AZI,VERTICAL ARRAY DIMENSION
C     U,V,W            - GUESS WIND FIELD 
C     OLAT,OLON        - GUESS WIND LAT,LON OF GRID ORIGIN
C     SX,SY,SZ         - GUESS WIND RADIAL,AZI,VERTICAL GRID SPACING (KM,DEG,KM)
C     XZ,YZ,ZZ         - GUESS WIND MIN RADIUS,AZI,HEIGHT (KM,DEG,KM)
C     ZLOW  - ABOVE MELTING BAND REFLECTIVITY BELOW ZLOW MEANS ALL SNOW
C     ZHIGH - ABOVE MELTING BAND REFLECTIVITY ABOVE ZHIGH MEANS ALL RAIN
C     HB2              - HEIGHT OF CENTER OF MELTING BAND (KM)
C     DPB2             - THICKNESS OF MELTING LAYER
C     ZLOW,ZHIGH,HB2,DPB2 WILL BE USED TO ADD FALL SPEED TO GUESS WIND FIELD
C          TO MAKE GUESS FIELD OF PRECIPITATION MOTION
C
C     SPECIFICATIONS FOR LOCAL VARIABLES
C     LULDPLN IS THE UNIT NUMBER OF THE INPUT GUESS WIND FIELD
C     IUV=0 FOR TRADITIONAL 2-BYTE INTEGER FILE AND 1 FORE FULLY 4-BYTE DATA
C
      INTEGER LULDPLN,IUV
      INTEGER KOUNT,NMOSM,IUNFLD,IATTEN,FLAG,EXTRA2,EXTRA3
      REAL STIME,ETIME,ROT,RA,CO1,CO2,AZMCOR,ELCOR,THRESH,POWERT,
     + BIEL,AZBIEL,ETIME1,STIME2,EXTRA6,EXTRA7
      CHARACTER KEYWORD*4,FLTNAME*8,STMNAME*12,RADAR*4,EXPERIMENT*32
      CHARACTER CREATIME*32,EXTRA1*28,FILNAM(40)*56,OFILNAM*56
      CHARACTER TESTFILE*56,KEYWORDOUT*4,WINDFILE*80 
C
C ... DESCRIPTION OF VARIABLES PASSED BETWEEN SUBROUTINES
C 
C **** READHEADERS

C **** LDPLN

C 
      WRITE(6,*)'Enter mask wind file name'
      WINDFILE='first_guess_fourier_xy.w'
C      READ(5,'(A80)')WINDFILE
C 
      LULDPLN=99
C 
      write(6,*)'calling readheaders'
      CALL READHEADERS(LULDPLN,WINDFILE,KEYWORD,FLTNAME,
     +   STMNAME,RADAR,EXPERIMENT,
     +   CREATIME,EXTRA1,IMX,JMX,KMX,KOUNT,NMOSM,
     +   IUNFLD,IATTEN,FLAG,EXTRA2,EXTRA3,STIME,ETIME,OLAT,OLON,
     +   SX,SY,SZ,XZ,YZ,ZZ,ROT,RA,CO1,CO2,AZMCOR,ELCOR,THRESH,
     +   POWERT,BIEL,AZBIEL,ETIME1,STIME2,EXTRA6,EXTRA7)
C 
      write(6,*)'returned from readheaders'
      WRITE(6,*)'EXTRA1,IMX,JMX,KMX,KOUNT,NMOSM,IUNFLD,IATTEN'
      WRITE(6,*)EXTRA1,IMX,JMX,KMX,KOUNT,NMOSM,IUNFLD,IATTEN
      WRITE(6,*)'EXTRA3,STIME,ETIME,OLAT,OLON,SX,SY,SZ,XZ,YZ,ZZ'
      WRITE(6,*)EXTRA3,STIME,ETIME,OLAT,OLON,SX,SY,SZ,
     +          XZ,YZ,ZZ
      WRITE(6,*)'SUBTRACT WIND FIELD FROM VR YES(1)/NO(0)?'
C 
      IUV=0
C 
C THE LDPLN CALLED FROM THIS ROUTINE ADDS THE FALL VELOCITY SO
C ITS PROJECTION CAN BE COMPARED TO THE DOPPLER RADIAL VELOCITY
C
      CALL LDPLN(LULDPLN,U,V,W,IMX,JMX,KMX,IUV,ZZ,SZ,
     + ZLOW,ZHIGH,HB2,DPB2)
C 
      CLOSE(LULDPLN)
C 
      RETURN
      END

