/* Changed variables frpm aampt_* to aamps_* to conform with
* latest from SIGMET/Vaisala. 5 Aug 2008
  16 Oct 2008: Malibu filed to favricate suitable radar
  antennae so they defaulted on the contract. So intul we get
  updated stuff from Sigmet I nhanged all occcurrences of 
  Malibu to the non-denominational TDR_hdr
*/
#ifndef TDR_HDR_H
#define TDR_HDR_H

#define CM_PER_FT (30.48)
#define KM_PER_NM (1.852)

#define CMS_PER_FTM (CM_PER_FT / 60.0)
#define CMS_PER_KNOT (KM_PER_NM * 100000.0/3600.0)


/**  Binary Angle of Measurement(BAM) [-180 deg, 180 deg) */
typedef int16_t bam16_t;
typedef int32_t bam32_t;

/* The Ansi C compiler on the HRD HP machines. Inline functions are supported
 * by C++, gcc and C99, so inline functions probably will be supported on the
 * NOAA airborne workstations. So I will put these inline functions in 
 * iris_functions.c and put conditional compile flads here and there. P Dodge,
 * 22 Feb 2008
 */
#define USE_INLINE 0
 

#if USE_INLINE /* Start of inline functions */

inline static bam32_t degTObam32(double deg)
{
    return (bam32_t)(INT32_MIN*(deg/-180.0));
}

inline static double bam32TOdeg(bam32_t bam)
{
    return  (-180.0) * (double)bam / (double)(INT32_MIN);
}

inline static bam16_t degTObam16(double deg)
{
    return (bam16_t)(INT16_MIN*(deg/-180.0));
}

inline static double bam16TOdeg(bam16_t bam)
{
    return (-180.0) * (double)bam / (double)(INT16_MIN);
}

#endif /* End of inline functions */
/* To make the numner of bytes in the extemded header data match the 
 * sizeof( struct malibu_header) I added three initial wlwmenys from the
 * Sigmet docs. 25 Feb 2008
 */
struct TDR_header {
    int32_t time_since_sweep_start;
    int16_t cal_sig_level;	int16_t num_bytes;
    uint16_t __pad_1;
    uint16_t gps_age;   /**< Time in milliseconds since last GPS Input */
    uint16_t irs_age;   /**< Time in milliseconds since last IRS Input */
    uint16_t aamps_age; /**< Time in milliseconds since last AAMPS Input */
    bam32_t  gps_lat;   /**< GPS latitude (BAM) */
    bam32_t  gps_long;  /**< GPS Longitude (BAM) */
    int32_t  gps_alt;   /**< GPS Altitude (?cm?ft?) */
    int32_t  gps_vel_e; /**< GPS Ground Speed East (cm/second) */
    int32_t  gps_vel_n; /**< GPS Ground Speed North (cm/second) */
    int32_t  gps_vel_v; /**< GPS Ground Speed Up (cm/second) */
    bam32_t  irs_lat;   /**< IRS latitude (BAM) */
    bam32_t  irs_long;  /**< IRS Longitude (BAM) */
    int32_t  irs_vel_e; /**< IRS Ground Speed East (cm/second) */
    int32_t  irs_vel_n; /**< IRS Ground Speed North (cm/second) */
    int32_t  irs_vel_v; /**< IRS Ground Speed Up (cm/second) */
    bam16_t  irs_pitch;     /**< IRS Pitch (BAM) */
    bam16_t  irs_roll;      /**< IRS Roll (BAM) */
    bam16_t  irs_heading;   /**< IRS Heading (BAM) */
    bam16_t  irs_drift;     /**< IRS Drift (BAM) */
    bam16_t  irs_tru_track; /**< IRS True Track (BAM) */
    bam16_t  irs_pitch_r;   /**< IRS Pitch rate (BAM/sec) */
    bam16_t  irs_roll_r;    /**< IRS Roll rate (BAM/sec) */
    bam16_t  irs_yaw_r;     /**< IRS Yaw rate (BAM/sec) */
    int32_t  irs_wind_vel;  /**< IRS Wind speed (cm/second) */
    bam16_t  irs_wind_dir;  /**< IRS Wind direction (BAM) */
    uint16_t __pad_2;
    bam32_t  aamps_lat;   /**< AAMPT latitude (BAM) */
    bam32_t  aamps_long;  /**< AAMPT Longitude (BAM) */
/* fixed so alt not a binary angle 14 Jan 1009 */
    int32_t  aamps_alt;   /**< AAMPT Altitude (cm) */
/* replaced u,v with speen and time stamp
    int32_t  aamps_vel_e; 
    int32_t  aamps_vel_n; 
 */
    int32_t  aamps_ground_vel; /**< AAMPS Ground Speed East (cm/second) */
    int32_t  time_stamp;       /**< AAMPS Timestamp in UTC (seconds since the epoch) */

    int32_t  aamps_vel_v; /**< AAMPT Ground Speed Up (cm/second) */
    bam16_t  aamps_pitch;     /**< AAMPT Pitch (BAM) */
    bam16_t  aamps_roll;      /**< AAMPT Roll (BAM) */
    bam16_t  aamps_heading;   /**< AAMPT Heading (BAM) */
    bam16_t  aamps_drift;     /**< AAMPT Drift (BAM) */
    bam16_t  aamps_track;     /**< AAMPT Track (BAM) */
/* replaced radar alt status with pad wors 14 Jan 09 */
/*    uint16_t aamps_radalt_stat; */
    uint16_t __pad_4;
    int32_t  aamps_radalt_val;  /**< AAMPT altitude (cm) */
/* /changed these 14 Jan 2009. Hope there are no more f***inf changes. */
/*
    int32_t  aamps_wind_vel_e; 
    int32_t  aamps_wind_vel_n; 
    int32_t  aamps_wind_vel_v; 
*/
    int32_t  aamps_wind_vel;    /**< AAMPS Wind Speed (cm/second) */
    bam16_t  aamps_wind_dir;    /**< AAMPS Wind direction (BAM) */
    uint16_t __pad_5;    
    int32_t  aamps_wind_vel_v;  /**< AAMPS Wind Speed Up (cm/second) */

};

/* ARINC 429 uses bit SDI bits for data
 * BNR: bits 28-9 data, bit 29 parity
 * BCD: bits 29-9 data.
 */
#define ARINC_LABEL_BIT_POS   0
#define ARINC_DATA_BIT_POS    8  //ARINC 429 uses bit SDI bits for data
#define ARINC_SSM_BIT_POS    29
#define ARINC_P_BIT_POS      31

#define ARINC_LABEL_MASK  0x000000ff
#define ARINC_DATA_MASK   0x001fffff
#define ARINC_SSM_MASK    0x00000003
#define ARINC_P_MASK      0x00000001


/** Channel 1 (IRS) Octal Labels */
enum arinc429_octal {
    OCT_TK_ANGLE_TRUE_BCD       = 0013,
    OCT_POS_LAT_BNR             = 0310,
    OCT_POS_LON_BNR             = 0311,
    OCT_TRUE_HDG_BNR            = 0314,
    OCT_WIND_SPEED_BNR          = 0315,
    OCT_WIND_DIR_BNR            = 0316,
    OCT_DRIFT_ANGLE_BNR         = 0321,
    OCT_PITCH_ANGLE_BNR         = 0324,
    OCT_ROLL_ANGLE_BNR          = 0325,
    OCT_BODY_PITCH_RATE_BNR     = 0326,
    OCT_BODY_ROLL_RATE_BNR      = 0327,
    OCT_BODY_YAW_RATE_BNR       = 0330,
    OCT_INERTIAL_VERT_SPEED_BNR = 0365,
    OCT_IRS_N_S_VEL_BNR         = 0366,
    OCT_IRS_E_W_VEL_BNR         = 0367
};

/** Channel 2 (GPS) Octal Labels */
enum arinc429_gps_octal {
    OCT_GPS_ALT_BNR             = 0076,
    OCT_GPS_LAT_BNR             = 0110,
    OCT_GPS_LON_BNR             = 0111,
    OCT_GPS_LAT_FINE_BNR        = 0120,
    OCT_GPS_LON_FINE_BNR        = 0121,
    OCT_GPS_VERT_VEL_BNR        = 0165,
    OCT_GPS_N_S_VEL_BNR         = 0166,
    OCT_GPS_E_W_VEL_BNR         = 0174
};

/* Fuhction prototype */
void get_TDR_hdr(FILE *, struct TDR_header *, int, int);

#endif /* MALIBU_H */
