/*   Version of iris_structures.h that includes the extended header
 * designed by Sigmet-Vaisala for the G IV Doppler radar. 
 * This subset of IRIS [copyright SIGMET] should be sufficient to read
 * raw format files on disk or from tape. See the latest version of the
 * IRIS Programmer's Manual at www.sigmet.com for details. Note that my 
 * variable names probably do NOT conform to those in the official IRIS
 * *.h files...but if you have IRIS you should not need my code anyway!
 * 25 May 2004
 * P dodge, HRD/AOML/OAR/NOAA
 * added stuff 5 Jan 2005
 */
# include "sigmet_dsp_data_types.h"
# define BLK_LENGTH 3072	/* 6144 bytes in raw data blocks */
# define MAX_WDS 2000 		/* Max 4000 bytes in a single ray */
# define MAX_DATA 4000		/* Max 4000 data words - maybe overkill*/
# define ANG_CONV 180.0/32768.0 /* 16 bit binary angle */
# define DOUBLE_ANG_CONV 360.0/4294967296.0 /* 32 bit binary angle */
# define ANG_CON_16 -180.0/INT16_MIN
# define ANG_CON_32 -180.0/INT32_MIN

# define TRUE 1
# define FALSE 0

/* 
 * Structures
 */
struct struct_hdr {
		int short struct_id;
		int short format_version_num;
		int num_bytes;
		int short reserved;
		int short flags;
	};

struct ymds_time {
		int seconds;
		int short msec; /* in lowest 10 bits */
		int short year;
		int short month;
		int short day;
	};

struct raw_psi_struct {
		int data_bit_mask;
		int last_bin_rng;
		int format_conv_flag;
		int flag;
		int sweep_number;
		int spare[6];
	};

/* First record on tape is 320 bytes, with this at beginning:
 */
struct tape_header_rec {
		struct struct_hdr struct_hdr;
		char tape_id[16];
		char site_name[16];
		struct ymds_time create_time;
		int spare;
		char iris_version[8];
		int spare2[63];
	};

/* Rest of data in files, one sweep per file.
 * First record holds product_hdr:
 * struct_hdr + product_configuration + product_end
 */
struct product_config {
		struct struct_hdr struct_hdr;
		int short product_type_code;
		int short sched_code;
		int skip_seconds;
		struct ymds_time prod_gen_time;
		struct ymds_time swp_ingest_time;
		struct ymds_time file_ingest_time;
		char spare[6];
		char prod_config_file[12];
		char task_name[12];
		int short flag_word;
		int x_scale;
		int y_scale;
		int z_scale;
		int x_array_size;
		int y_array_size;
		int z_array_size;
		int x_radar_loc;
		int y_radar_loc;
		int z_radar_loc;
		int maximum_rng;
		char spare2[2];
		int short data_type_gen;
		char projection[12];
		int short data_type_input;
		char proj;
		char spare3;
		int short rad_smoother;
		int short num_run_times;
		int z_r_const;
		int z_r_exp;
		int short x_smooth;
		int short y_smooth;
/* Note following can hold many different types of structures [enum?];
 * For files on tape, though, it should only be raw_psi_struct [44 bytes],;
 *  so that is what I will declare here:;
 */
		struct raw_psi_struct raw_prod_info;
		int short spare4[18];
/* Then 28 char spare and 48 char color scale def, but that may be;
 * removed in future, so iset it spare also...;
 */
		int short spare5[14];
		int short spare6[24];
	};
	
struct product_end {
		char prod_site_name[16];
		char iris_version_prod[8];
		char iris_version_ingest[8];
		struct ymds_time oldest_time;
		char spare[28];
		int short local_ahead_gmt;
		char hardware_name[16];
		char ingest_site_name[16];
		int short ahead_gmt;
		int lat_center;
		int lon_center;
		int short gd_height;
		int short radar_ht_above_gd;
		int prf;
		int pulse_width;
		int short sig_proc_type;
		int short trigger_scheme;
		int short num_samples;
		char clutter_file[12];
		int short filter_num;
		int wavelength;
		int trunc_height;
		int first_bin_rng;
		int last_bin_rng;
		int num_output_bins;
		int short flag_word;
		int short num_files_used;
		int short polarization_used;
		char spare2[54];
		int task_fault_status;
		int input_sites_mask;
		int short log_filter_num;
		int short cluttermap_flag;
		int lat_proj_ref;
		int lon_proj_ref;
		int short prod_seq_num;
		int short color_numbers[16];
		int short spare3;
		int short ref_height;
		int short num_array_elem;
		char mean_wind_speed;
		char mean_wind_dir;
		char spare4[2];
		char time_zone_name[8];
		char spare5[8];
	};

/*  Then the second record holds the ingest_header =;
 * struct_hdr + ingest_configuration + task_configuration +;
 * 732 char spare + GPARM from DSP1 [524 bytes].;
 */
/* Modified 5 March 2008 to conform to latest IRIS docs
 */
 struct ingest_configuration {
		char disk_file[80];
		int short num_assoc_files;
		int short spare;
		int tot_file_size;
		struct ymds_time vscan_start_time;
		char spare2[12];
		int short num_bytes_ray_hdr;
		int short n_bytes_ext_hdr;
		int short num_task_c_tab;
		char spare3[4];
		char iris_version[8];
		char hardware_site_name[16];
		int short time_zone_lst;
		char site_name[16];
		int short time_zone_recorded;
		int radar_lat;
		int radar_lon;
		int short gd_height;
		int short radar_ht_above_gd;
		int short resolution;
		int short index_first_ray;
		int short n_rays_sweep;
		int short n_bytes_gparam;
		int radar_ht_msl;
		int platform_vel[3];
		int antenna_inu_offset[3];
		int start_fault_status;
		int short melt_layer_ht;
		int short spare6;
		char time_zone_name[8];
		int flags;
		char config_name[16];
		int short spare5[114];
	};

/* The next structure is task_configuration:
 * struct_hdr + task_sched_info + task_dsp_info + task_calib_info +;
 * task_range_info + task_scan_info + task_misc_info + task_end_info + ;
 * 730 bytes of comments!
 */
struct task_sched_info {
		int start_sec;
		int stop_sec;
		int skip_sec;
		int last_run_time_sec;
		int last_time_used;
		int last_run_day;
		int short flag;
		int short spare[47];
	};

struct task_dsp_info {
		int short major_mode;
		int short dsp_type;
		int data_type_mask[6]; 
		int aux_data_def[27];
		int prf;
		int pulse_width;
		int short prf_mode;
		int short dual_prf_delay;
		int short agc_feedback_code;
		int short sample_size;
		int short gain_control_flag;
		char clutter_filter_file[12];
		char lin_filter_num;
		char log_filter_num;
		int short attenuation;
		int short gas_attenuation;
		int short clutter_flag;
		int short xmt_phase_seq;
/* this last spare probably contains the name of the custom extended 
 * headet in a 16 character ASCII string. 5 Feb 2008
 */
		int short spare2[72];
	};

struct task_calib_info {
		int short refl_slope;
		int short refl_noise_thresh;
		int short clutter_corr_thresh;
		int short sqi_threshold;
		int short pwr_trhreshold;
		int short spare[4];
		int short cal_refl;
		int short uncor_refl_thresh;
		int short cor_refl_thresh;
		int short vel_thresh;
		int short width_thresh;
		int short zdr_thresh;
		int short spare2[3];
		int short flags;
		int short spare3[2];
		int short zdr_bias;
		int short spare4[138];
	};

struct task_range_info {
		int first_bin_range;
		int last_bin_range;
		int short num_input_bins;
		int short num_output_bins;
		int input_step;
		int output_step;
		int short flag;
		int short rng_avg_flag;
		int short spare[68];
	};

struct task_scan_info {
		int short antenna_scan_mode;
		int short ang_resolution;
		int short spare;
		int short num_sweeps;
/* The following 200 bytes can hold a variety of structures:;
 * task_rhi_scan_info, task_ppi_scan_info, task_file_scan, or;
 * task_manual_scan_info:
 */
		int short scan_info[100];
		int short spare2[56];
	};

struct task_misc_info {
		int wavelength;
		char tr_ser_number[16];
		int trans_power;
		int short flags;
		int short polarization_type;
		int trunc_height;
		int short spare_pol_spec[9];
		int short spare[6];
		int short n_bytes_comments;
		int horiz_beamwidth;
		int vert_beamwidth;
		int customer_defined[10];
		int short spare2[104];
	};

struct task_end_info {
		int short task_major_num;
		int short task_minor_num;
		char task_config_file[12];
		char task_description[80];
		int num_tasks;
		int short task_state;
		int short spare;
		struct ymds_time data_time;
		int short spare2[102];
	};
	
/* Well, the damn task_scan_info can hold one of the following:;
 * task_rhi_scan_info, task_ppi_scan_info, task_file_scan_info, or;
 * task_manual_scan_info. So I guess I need those also, because I don't;
 * know if I am going to get PPI's, RHI's....
 */
 
struct task_rhi_scan_info {
		int short el_start_angle;
		int short el_end_angle;
		int short az_scan_angles[40];
		int short spare[58];
	};

struct task_ppi_scan_info {
		int short az_start_angle;
		int short az_end_angle;
		int short el_scan_angles[40]		;
		int short spare[58];
	};

struct task_file_scan_info {
		int short first_az_angle;
		int short first_el_angle;
		char ant_ctrl_file[12];
		int short spare[92];
	};

struct task_manual_scan_info {
		int short flag;
		int short spare[99];

/* the other 198 bytes are spare
 */
	};

/* Then, after we've waded through all that stuff, we finally get to the data.;
 * Well, almost. First each record starts with raw_prod_bhdr, a header for raw ;
 * data, and if is the first data record there will be ingest_data_headers,;
 * one for each type of data recorded. Then aach ray in the record starts with ;
 * -32762 [=6 data words to follow], then the six words of the ray header, and ;
 * then the run_length encoded data. Whew!
 */

struct ingest_data_hdr {
	struct struct_hdr s_hdr;
	struct ymds_time date;
	int short sweep_num;
	int short ray_resolution;
	int short first_ray_index;
	int short expected_num_rays;
	int short num_rays_in_file;
	unsigned int short fixed_angle;
	int short bits_per_bin;
	unsigned int short data_type;
	int short spare[18];
};

struct raw_prod_bhdr {
	int short rec_num;
	int short sweep_num;
	int short offset_first_full_ray;
	int short ray_num;
	unsigned int short flags;
	int short spare;
};

struct ray_hdr {
	unsigned int short az_beg;
	unsigned int short el_beg;
	unsigned int short az_end;
	unsigned int short el_end;
	int short num_bins_in_ray;
	unsigned int short secs_from_start;
};

struct extended_hdr_v0 {
	int msec_from_start;
	int short cal_sig_level;
	int short spare[7];
};

struct extended_hdr_v1 {
	int msec_from_start;
	int short cal_sig_level;
	unsigned int short az;
	unsigned int short el;
	unsigned int short train_order;
	unsigned int short el_order;
	unsigned int short pitch;
	unsigned int short roll;
	unsigned int short heading;
	unsigned int short az_rate;
	unsigned int short el_rate;
	unsigned int short pitch_rate;
	unsigned int short roll_rate;
	unsigned int lat;
	unsigned int lon;
	unsigned int short hdg_rate;
	int short altitude;
	int short vel_ew;
	int short vel_ns;
	int msec_since_update;
	int short vel_vert;
	unsigned int short nav_sys_ok_flag;
	int short radial_vel_correction;
};

/*
 * Function prototypes for iris_functions.c
 */
int get_short(int short *, FILE *);
void get_struct_hdr(FILE *, struct struct_hdr *, int , int);
void get_ingest_config(FILE *, struct ingest_configuration *,
	        int , int);
void get_raw_prod_bhdr(FILE *, struct raw_prod_bhdr *,
	        int , int);
void get_ingest_data_hdr(FILE * , struct ingest_data_hdr *,
	        int, int);
int get_ray_hdr(FILE *, struct ray_hdr *, int, int);
int get_ray(FILE *, struct ray_hdr *, 
		struct raw_prod_bhdr *, 
		int short ray_word[MAX_WDS],
	        int , int , int *, int *);
void expand_ray( int short ray_word[MAX_WDS],
		float ray_data[MAX_DATA], int , int , int short, 
	        unsigned int short, float, int *);
/*
void iris_word_to_float( int short, float *, int short, float, int *);
*/
